//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : *(PSM3 TARGET SYSTEM)
//  FUNCTION      : Clock & Reset Generator of Psm3DualCoreSystem
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//=======================================================================
//============ Tclk = 50MHz ===================
//`define     BPS115200   5'd26
//`define     BPS56000    6'd53
//`define     BPS38400    7'd80
//`define     BPS19200    8'd161
//`define     BPS9600     9'd324

//============ Tclk = 70MHz ===================
`define     BPS115200   6'd38
`define     BPS56000    7'd78
`define     BPS38400    7'd114
`define     BPS19200    8'd228
`define     BPS9600     9'd455

//============ Tclk = 100MHz ===================
`define     BPS230400   5'd26


module ClkRstGen(
    //======== input ================
    CK50M           ,
    CLK_AUX         ,
    CLK_SMA         ,
    RESET           ,

    FX2_CLKIO       ,

    //======== output ===============
    Clk             ,
    Clk100m         ,
    Clk100mP270     ,
    RstN            ,
    DdrRstN         ,

    Clk16x230k4     ,
    Clk16x115k2     ,
    Clk16x56k       ,
    Clk16x38k4      ,
    Clk16x19k2      ,
    Clk16x9k6
);
    //============================================================
    //======== port  =============================================
    //============================================================
    //======== input ========================
    input           CK50M           ; // Clock Input(50MHz)
    input           CLK_AUX         ; // RESERVE
    input           CLK_SMA         ; // RESERVE
    input           RESET           ; // RESET : PushSW[2] (SOUTH)
    output          FX2_CLKIO       ;

    //======== output =======================
    output          Clk             ; // Master Clock (70MHz)
    output          Clk100m         ; // Ddr Masr Clock (100MHz)
    output          Clk100mP270     ; // Ddr Masr Clock (100MHz P270)
    output          RstN            ; // Master Reset
    output          DdrRstN         ; // Ddr Reset

    output          Clk16x230k4     ; // for uart 230400x16Hz
    output          Clk16x115k2     ; // for uart 115200x16Hz
    output          Clk16x56k       ; // for uart  56000x16Hz
    output          Clk16x38k4      ; // for uart  38400x16Hz
    output          Clk16x19k2      ; // for uart  19200x16Hz
    output          Clk16x9k6       ; // for uart   9600x16Hz

    //============================================================
    //======== wire & reg ========================================
    //============================================================
    //wire            Clk100mTmp      ;
    wire            Dcm1Locked      ;
    wire            Dcm2Locked      ;

    reg             Clk16x230k4     ;
    reg             Clk16x115k2     ;
    reg             Clk16x56k       ;
    reg             Clk16x38k4      ;
    reg             Clk16x19k2      ;
    reg             Clk16x9k6       ;

    reg     [4:0]   BCounter0       ;
    reg     [6:0]   BCounter2       ;
    reg     [2:0]   BCounter0Edge   ;
    reg             Tff115k2         ;
    reg             Tff56k          ;
    reg             OneDly38k2      ;
    reg             Tff19k2         ;
    reg             Tff9k6          ;
    reg     [1:0]   SyncRstN        ;
    reg     [3:0]   RstCount        ;
    wire            AuxClk          ;


    //===========================================================================
    //======== BODY  ============================================================
    //===========================================================================
    //======== RESET =============================================
    //assign RstN = Dcm2Locked;
    always @(posedge Clk or posedge RESET)
        if(RESET) SyncRstN <= 2'b00;
        else      SyncRstN <= {SyncRstN[0], Dcm2Locked};
    assign RstN = SyncRstN[1];
    assign DdrRstN = Dcm2Locked;

    always @(posedge AuxClk or posedge RESET)
        if(RESET)                 RstCount <= 4'hF;
        else if(RstCount != 4'h0) RstCount <= RstCount - 1'b1;


    //======== dcm1 =============================================
    dcm1 dcm1(
        .CLKIN_IN           ( CK50M         ),
        .RST_IN             ( RESET         ),
        .CLKIN_IBUFG_OUT    (               ),
        .CLK0_OUT           (               ),
        .CLKFX_OUT          ( Clk           ),
        .CLK2X_OUT          ( FX2_CLKIO     ),
        .LOCKED_OUT         ( Dcm1Locked    )
    );


    //======== dcm2 =============================================
    dcm2 dcm2(
        .CLKIN_IN           ( CLK_AUX       ),
        .RST_IN             ( |RstCount     ),
        .CLKDV_OUT          (               ),
        .CLKIN_IBUFG_OUT    ( AuxClk        ),
        .CLK0_OUT           ( Clk100m       ),
        .CLK270_OUT         ( Clk100mP270   ),
        .LOCKED_OUT         ( Dcm2Locked    )
    );

    //===========================================================
    //======== UartClockGen =====================================
    //===========================================================

    //======== Clk16x230k4 =====================================
    always @(posedge Clk100m or negedge DdrRstN) begin
        if(~DdrRstN)                 BCounter0 <= 5'h0;
        else if(BCounter0 == 5'h0)   BCounter0 <= `BPS230400;
        else                         BCounter0 <= BCounter0 - 1'b1;
    end
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)   BCounter0Edge <= 3'h0;
        else        BCounter0Edge <= {BCounter0Edge[1:0], BCounter0[4]};
    end
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)   Clk16x230k4 <= 1'h0;
        else        Clk16x230k4 <= {BCounter0Edge[2:1] == 2'b01};
    end
    //======== Clk16x115k2 =====================================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)               Tff115k2  <= 1'h0;
        else if(Clk16x230k4)    Tff115k2  <= ~Tff115k2;
    end
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)   Clk16x115k2 <= 1'h0;
        else        Clk16x115k2 <= Tff115k2 & Clk16x230k4;
    end
    //======== Clk16x56k =====================================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)               Tff56k  <= 1'h0;
        else if(Clk16x115k2)    Tff56k  <= ~Tff56k;
    end
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)   Clk16x56k   <= 1'h0;
        else        Clk16x56k   <= Tff56k & Clk16x115k2;
    end

    //======== Clk16x38k4 =====================================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)                    BCounter2 <= 7'h0;
        else if(BCounter2 == 7'h0)   BCounter2 <= `BPS38400 ;
        else                         BCounter2 <= BCounter2 - 1'b1;
    end
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)   OneDly38k2 <= 1'h0;
        else        OneDly38k2 <= BCounter2[6];
    end
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)   Clk16x38k4 <= 1'h0;
        else        Clk16x38k4 <= ({OneDly38k2, BCounter2[6]} == 2'b01);
    end

    //======== Clk16x19k2 =====================================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)           Tff19k2 <= 1'h0;
        else if(Clk16x38k4) Tff19k2 <= ~Tff19k2;
    end
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)   Clk16x19k2  <= 1'h0;
        else        Clk16x19k2  <= Tff19k2 & Clk16x38k4;
    end

    //======== Clk16x9k6 =====================================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)           Tff9k6  <= 1'h0;
        else if(Clk16x19k2) Tff9k6  <= ~Tff9k6;
    end
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)   Clk16x9k6   <= 1'h0;
        else        Clk16x9k6   <= Tff9k6 & Clk16x19k2;
    end

endmodule

