//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : Bus Arbiter for DDR-SDRAM I/F
//  VERSION       : Ver0.70
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//=======================================================================

`define     DDR_ABT_IDLE    2'h0
`define     DDR_ABT_W1      2'h1
`define     DDR_ABT_W2      2'h2
`define     DDR_ABT_W3      2'h3

//====================================================================
//======== DdrAbt ====================================================
//====================================================================
module DdrAbt(
    //======== SYSTEM ==============
    Clk             ,
    RstN            ,

    //======= Master0 Bus[Read:32bit/Write:32bit] ================
    Mstr0CRQ        ,
    Mstr0CAK        ,
    Mstr0BRST       ,
    Mstr0RW         ,
    Mstr0BE         ,
    Mstr0ADR        ,
    Mstr0WD         ,
    Mstr0WDAK       ,
    Mstr0RD         ,
    Mstr0RDAK       ,

    //======= Master1 Bus[Read:32bit/Write:32bit] ================
    Mstr1CRQ        ,
    Mstr1CAK        ,
    Mstr1BRST       ,
    Mstr1RW         ,
    Mstr1BE         ,
    Mstr1ADR        ,
    Mstr1WD         ,
    Mstr1WDAK       ,
    Mstr1RD         ,
    Mstr1RDAK       ,

    //======= Master2 Bus[Read:32bit/Write:32bit] ================
    Mstr2CRQ        ,
    Mstr2CAK        ,
    Mstr2BRST       ,
    Mstr2RW         ,
    Mstr2BE         ,
    Mstr2ADR        ,
    Mstr2WD         ,
    Mstr2WDAK       ,
    Mstr2RD         ,
    Mstr2RDAK       ,

    //======= Master3 Bus[Read:32bit/Write:32bit] ================
    Mstr3CRQ        ,
    Mstr3CAK        ,
    Mstr3BRST       ,
    Mstr3RW         ,
    Mstr3BE         ,
    Mstr3ADR        ,
    Mstr3WD         ,
    Mstr3WDAK       ,
    Mstr3RD         ,
    Mstr3RDAK       ,

    //======= Master4 Bus[Read:32bit/Write:32bit] ================
    Mstr4CRQ        ,
    Mstr4CAK        ,
    Mstr4BRST       ,
    Mstr4RW         ,
    Mstr4BE         ,
    Mstr4ADR        ,
    Mstr4WD         ,
    Mstr4WDAK       ,
    Mstr4RD         ,
    Mstr4RDAK       ,

    //======= Master5 Bus[Read:32bit/Write:32bit] ================
    Mstr5CRQ        ,
    Mstr5CAK        ,
    Mstr5BRST       ,
    Mstr5RW         ,
    Mstr5BE         ,
    Mstr5ADR        ,
    Mstr5WD         ,
    Mstr5WDAK       ,
    Mstr5RD         ,
    Mstr5RDAK       ,

    //======= Command Fifo & Read Data Fifo =============
    DdrComFifoFull  ,
    DdrComFifoWe    ,
    DdrComMstr      ,
    DdrBRST         ,
    DdrRW           ,
    DdrBE           ,
    DdrADR          ,
    DdrWD           ,

    DdrRdFifoEmpty  ,
    DdrRdFifoRe     ,
    DdrRDMstr       ,
    DdrRD
);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           Clk             ;
    input           RstN            ;

    //======= Master0 Bus[Read:32bit/Write:32bit] ================
    input           Mstr0CRQ        ;
    output          Mstr0CAK        ;
    input           Mstr0BRST       ;
    input           Mstr0RW         ;
    input   [3:0]   Mstr0BE         ;
    input   [25:2]  Mstr0ADR        ;
    input   [31:0]  Mstr0WD         ;
    output          Mstr0WDAK       ;
    output  [31:0]  Mstr0RD         ;
    output          Mstr0RDAK       ;

    //======= Master1 Bus[Read:32bit/Write:32bit] ================
    input           Mstr1CRQ        ;
    output          Mstr1CAK        ;
    input           Mstr1BRST       ;
    input           Mstr1RW         ;
    input   [3:0]   Mstr1BE         ;
    input   [25:2]  Mstr1ADR        ;
    input   [31:0]  Mstr1WD         ;
    output          Mstr1WDAK       ;
    output  [31:0]  Mstr1RD         ;
    output          Mstr1RDAK       ;

    //======= Master2 Bus[Read:32bit/Write:32bit] ================
    input           Mstr2CRQ        ;
    output          Mstr2CAK        ;
    input           Mstr2BRST       ;
    input           Mstr2RW         ;
    input   [3:0]   Mstr2BE         ;
    input   [25:2]  Mstr2ADR        ;
    input   [31:0]  Mstr2WD         ;
    output          Mstr2WDAK       ;
    output  [31:0]  Mstr2RD         ;
    output          Mstr2RDAK       ;

    //======= Master3 Bus[Read:32bit/Write:32bit] ================
    input           Mstr3CRQ        ;
    output          Mstr3CAK        ;
    input           Mstr3BRST       ;
    input           Mstr3RW         ;
    input   [3:0]   Mstr3BE         ;
    input   [25:2]  Mstr3ADR        ;
    input   [31:0]  Mstr3WD         ;
    output          Mstr3WDAK       ;
    output  [31:0]  Mstr3RD         ;
    output          Mstr3RDAK       ;

    //======= Master4 Bus[Read:32bit/Write:32bit] ================
    input           Mstr4CRQ        ;
    output          Mstr4CAK        ;
    input           Mstr4BRST       ;
    input           Mstr4RW         ;
    input   [3:0]   Mstr4BE         ;
    input   [25:2]  Mstr4ADR        ;
    input   [31:0]  Mstr4WD         ;
    output          Mstr4WDAK       ;
    output  [31:0]  Mstr4RD         ;
    output          Mstr4RDAK       ;

    //======= Master5 Bus[Read:32bit/Write:32bit] ================
    input           Mstr5CRQ        ;
    output          Mstr5CAK        ;
    input           Mstr5BRST       ;
    input           Mstr5RW         ;
    input   [3:0]   Mstr5BE         ;
    input   [25:2]  Mstr5ADR        ;
    input   [31:0]  Mstr5WD         ;
    output          Mstr5WDAK       ;
    output  [31:0]  Mstr5RD         ;
    output          Mstr5RDAK       ;


    //======= Command Fifo & Read Data Fifo =============
    input           DdrComFifoFull  ;
    output          DdrComFifoWe    ;
    output  [5:0]   DdrComMstr      ;
    output          DdrBRST         ;
    output          DdrRW           ;
    output  [3:0]   DdrBE           ;
    output  [25:2]  DdrADR          ;
    output  [31:0]  DdrWD           ;

    input           DdrRdFifoEmpty  ;
    output          DdrRdFifoRe     ;
    input   [5:0]   DdrRDMstr       ;
    input   [31:0]  DdrRD           ;


    //==================================================
    //======== wire & reg ==============================
    //==================================================
    wire            MstrCRQ         ;
    wire            MstrBRST        ;
    wire            MstrRW          ;
    wire    [3:0]   MstrBE          ;
    wire    [25:2]  MstrADR         ;
    wire    [31:0]  MstrWD          ;
    reg     [1:0]   DdrAbtState     ;
    wire    [5:0]   SelectedMstr0   ;
    reg     [5:0]   SelectedMstr    ;
    wire            DdrComFifoWe0   ;
    wire            DdrComFifoWe1   ;
    wire            DdrComFifoWe    ;
    reg             DdrRdFifoRe1d   ;

    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    //==================================================
    //======== DdrAbt State Machine  ===================
    //==================================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            DdrAbtState <= `DDR_ABT_IDLE;
        end
        else if(DdrAbtState == `DDR_ABT_IDLE) begin
            if(DdrComFifoWe0 & MstrRW & MstrBRST) DdrAbtState <= `DDR_ABT_W1;
        end
        else if(DdrAbtState == `DDR_ABT_W1) begin
            DdrAbtState <= `DDR_ABT_W2;
        end
        else if(DdrAbtState == `DDR_ABT_W2) begin
            DdrAbtState <= `DDR_ABT_W3;
        end
        else begin
            DdrAbtState <= `DDR_ABT_IDLE;
        end
    end

    //==================================================
    //======== Internal Signals  =======================
    //==================================================
    //======== DdrComMstr  =======================
    assign SelectedMstr0 = (Mstr0CRQ)? 6'b000001 :
                           (Mstr1CRQ)? 6'b000010 :
                           (Mstr2CRQ)? 6'b000100 :
                           (Mstr3CRQ)? 6'b001000 :
                           (Mstr4CRQ)? 6'b010000 :
                           (Mstr5CRQ)? 6'b100000 :
                                       6'b000000 ;
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                               SelectedMstr <= 6'h0;
        else if(DdrAbtState == `DDR_ABT_IDLE)   SelectedMstr <= SelectedMstr0;
    end
    assign  DdrComMstr = (DdrAbtState == `DDR_ABT_IDLE)? SelectedMstr0 : SelectedMstr;
    //======== DdrRW  =======================
    assign  MstrBRST   = (DdrComMstr[0])? Mstr0BRST:
                         (DdrComMstr[1])? Mstr1BRST:
                         (DdrComMstr[2])? Mstr2BRST:
                         (DdrComMstr[3])? Mstr3BRST:
                         (DdrComMstr[4])? Mstr4BRST:
                                          Mstr5BRST;
    assign  DdrBRST     = MstrBRST;
    //======== DdrRW  =======================
    assign  MstrRW     = (DdrComMstr[0])? Mstr0RW:
                         (DdrComMstr[1])? Mstr1RW:
                         (DdrComMstr[2])? Mstr2RW:
                         (DdrComMstr[3])? Mstr3RW:
                         (DdrComMstr[4])? Mstr4RW:
                                          Mstr5RW;
    assign  DdrRW      = MstrRW;

    //======== DdrBE  =======================
    assign  MstrBE    = (DdrComMstr[0])? Mstr0BE:
                        (DdrComMstr[1])? Mstr1BE:
                        (DdrComMstr[2])? Mstr2BE:
                        (DdrComMstr[3])? Mstr3BE:
                        (DdrComMstr[4])? Mstr4BE:
                                         Mstr5BE;
    assign  DdrBE     = MstrBE;
    //======== DdrADR  =======================
    assign  MstrADR  = (DdrComMstr[0])? Mstr0ADR:
                       (DdrComMstr[1])? Mstr1ADR:
                       (DdrComMstr[2])? Mstr2ADR:
                       (DdrComMstr[3])? Mstr3ADR:
                       (DdrComMstr[4])? Mstr4ADR:
                                        Mstr5ADR;
    assign  DdrADR  = MstrADR;

    //======== DdrWD  =======================
    assign  MstrWD   = (DdrComMstr[0])? Mstr0WD:
                       (DdrComMstr[1])? Mstr1WD:
                       (DdrComMstr[2])? Mstr2WD:
                       (DdrComMstr[3])? Mstr3WD:
                       (DdrComMstr[4])? Mstr4WD:
                                        Mstr5WD;
    assign  DdrWD    = MstrWD;

    //======== DdrComFifoWe ========================
    assign  MstrCRQ    = Mstr0CRQ  | Mstr1CRQ | Mstr2CRQ | Mstr3CRQ | Mstr4CRQ | Mstr5CRQ;
    assign  DdrComFifoWe0 = (DdrAbtState == `DDR_ABT_IDLE) & MstrCRQ & ~DdrComFifoFull;
    assign  DdrComFifoWe1 = (DdrAbtState != `DDR_ABT_IDLE);
    assign  DdrComFifoWe  = DdrComFifoWe0 | DdrComFifoWe1 ;

    //======== DdrRdFifoRe ==============================
    assign  DdrRdFifoRe = ~DdrRdFifoEmpty;
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) DdrRdFifoRe1d <= 1'b0;
        else      DdrRdFifoRe1d <= DdrRdFifoRe;
    end

    //======== MstrCAK ==============================
    assign  Mstr0CAK = DdrComFifoWe0 & SelectedMstr0[0];
    assign  Mstr1CAK = DdrComFifoWe0 & SelectedMstr0[1];
    assign  Mstr2CAK = DdrComFifoWe0 & SelectedMstr0[2];
    assign  Mstr3CAK = DdrComFifoWe0 & SelectedMstr0[3];
    assign  Mstr4CAK = DdrComFifoWe0 & SelectedMstr0[4];
    assign  Mstr5CAK = DdrComFifoWe0 & SelectedMstr0[5];

    //======== MstrWDAK ==============================
    assign  Mstr0WDAK = (Mstr0CAK & Mstr0RW) | (DdrComFifoWe1 & SelectedMstr[0]);
    assign  Mstr1WDAK = (Mstr1CAK & Mstr1RW) | (DdrComFifoWe1 & SelectedMstr[1]);
    assign  Mstr2WDAK = (Mstr2CAK & Mstr2RW) | (DdrComFifoWe1 & SelectedMstr[2]);
    assign  Mstr3WDAK = (Mstr3CAK & Mstr3RW) | (DdrComFifoWe1 & SelectedMstr[3]);
    assign  Mstr4WDAK = (Mstr4CAK & Mstr4RW) | (DdrComFifoWe1 & SelectedMstr[4]);
    assign  Mstr5WDAK = (Mstr5CAK & Mstr5RW) | (DdrComFifoWe1 & SelectedMstr[5]);

    //======== MstrRDAK ==============================
    assign  Mstr0RDAK = (DdrRdFifoRe1d & DdrRDMstr[0]);
    assign  Mstr1RDAK = (DdrRdFifoRe1d & DdrRDMstr[1]);
    assign  Mstr2RDAK = (DdrRdFifoRe1d & DdrRDMstr[2]);
    assign  Mstr3RDAK = (DdrRdFifoRe1d & DdrRDMstr[3]);
    assign  Mstr4RDAK = (DdrRdFifoRe1d & DdrRDMstr[4]);
    assign  Mstr5RDAK = (DdrRdFifoRe1d & DdrRDMstr[5]);

    //======== MstrRD ==============================
    assign  Mstr0RD = DdrRD;
    assign  Mstr1RD = DdrRD;
    assign  Mstr2RD = DdrRD;
    assign  Mstr3RD = DdrRD;
    assign  Mstr4RD = DdrRD;
    assign  Mstr5RD = DdrRD;

endmodule

