//============================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : IRQ CONTROLER
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//============================================================
`define     IRQ_MASK    2'h0
`define     IRQ_STS     2'h1
`define     IRQ_CLR     2'h2
`define     IRQ_SET     2'h3

module IrqCtrl(
    //======== SYSTEM ==============
    Clk         ,
    RstN        ,

    //======== Peripheral Bus ======
    HostCS      ,
    HostRQ      ,
    HostRW      ,
    HostADR     ,
    HostWD      ,
    HostRD      ,

    //======== from TIMER =========
    Timer0Irq   ,
    Timer1Irq   ,

    //======== from FifoUart =========
    UaSendIrq   ,
    UaRecvIrq   ,

    //======== Output ============
    IrqOut

);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           Clk     ;
    input           RstN    ;

    //======== Peripheral Bus ======
    input           HostCS  ;
    input           HostRQ  ;
    input           HostRW  ;
    input   [3:0]   HostADR ;
    input   [31:0]  HostWD  ;
    output  [31:0]  HostRD  ;

    //======== from TIMER =========
    input           Timer0Irq; // pulse
    input           Timer1Irq; // pulse

    //======== from FifoUart =========
    input           UaSendIrq; // pulse
    input           UaRecvIrq; // pulse

    //======== Output ============
    output          IrqOut  ;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    reg     [5:0]   IrqMask ;
    reg     [5:4]   SoftIrq ;
    reg             TmIrq0  ;
    reg             TmIrq1  ;
    reg             UaSIrq  ;
    reg             UaRIrq  ;
    wire            MyWe    ;
    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    assign MyWe = HostCS & HostRW & HostRQ;
    //======== Peripheral Bus ===================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                                   IrqMask <= 6'h3F;
        else if(MyWe & (HostADR[3:2] == `IRQ_MASK)) IrqMask <= HostWD[5:0];
    end
    //=============== TIMER0 ========================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                                                           TmIrq0   <= 1'b0;
        else if(Timer0Irq)                                                  TmIrq0   <= 1'b1;
        else if(MyWe & (HostADR[3:2] == `IRQ_CLR) & (HostWD[2:0] == 3'h0))  TmIrq0   <= 1'b0;
    end
    //=============== TIMER1 ========================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                                                           TmIrq1   <= 1'b0;
        else if(Timer1Irq)                                                  TmIrq1   <= 1'b1;
        else if(MyWe & (HostADR[3:2] == `IRQ_CLR) & (HostWD[2:0] == 3'h1))  TmIrq1   <= 1'b0;
    end
    //=============== UART SEND ========================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                                                           UaSIrq   <= 1'b0;
        else if(UaRecvIrq)                                                  UaSIrq   <= 1'b1;
        else if(MyWe & (HostADR[3:2] == `IRQ_CLR) & (HostWD[2:0] == 3'h2))  UaSIrq   <= 1'b0;
    end

    //=============== UART RECV ========================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                                                           UaRIrq   <= 1'b0;
        else if(UaSendIrq)                                                  UaRIrq   <= 1'b1;
        else if(MyWe & (HostADR[3:2] == `IRQ_CLR) & (HostWD[2:0] == 3'h3))  UaRIrq   <= 1'b0;
    end

    //=============== SoftIrq[4] ========================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                                                           SoftIrq[4]   <= 1'b0;
        else if(MyWe & (HostADR[3:2] == `IRQ_SET) & (HostWD[2:0] == 3'h4))  SoftIrq[4]   <= 1'b1;
        else if(MyWe & (HostADR[3:2] == `IRQ_CLR) & (HostWD[2:0] == 3'h4))  SoftIrq[4]   <= 1'b0;
    end

    //=============== SoftIrq[5] ========================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                                                           SoftIrq[5]   <= 1'b0;
        else if(MyWe & (HostADR[3:2] == `IRQ_SET) & (HostWD[2:0] == 3'h5))  SoftIrq[5]   <= 1'b1;
        else if(MyWe & (HostADR[3:2] == `IRQ_CLR) & (HostWD[2:0] == 3'h5))  SoftIrq[5]   <= 1'b0;
    end

    assign  HostRD = (HostADR[3:2] == `IRQ_MASK)? {26'h0, IrqMask} :
                     (HostADR[3:2] == `IRQ_STS )? {26'h0, SoftIrq, UaRIrq, UaSIrq, TmIrq1, TmIrq0} : 32'h0;

    //======== IrqOut ===================================
    assign  IrqOut = ((~IrqMask[5]) & SoftIrq[5])
                   | ((~IrqMask[4]) & SoftIrq[4])
                   | ((~IrqMask[3]) & UaRIrq    )
                   | ((~IrqMask[2]) & UaSIrq    )
                   | ((~IrqMask[1]) & TmIrq1    )
                   | ((~IrqMask[0]) & TmIrq0    );
endmodule
