//============================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : Peripherals Top Module
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//============================================================
module P3dPeriTop(
    //======== SYSTEM ==============
    Clk             ,
    VgaClk          ,
    RstN            ,

    Clk16x115k2     ,
    Clk16x56k       ,
    Clk16x38k4      ,
    Clk16x19k2      ,
    Clk16x9k6       ,

    //======= Peri Bus (Host0) ========
    PeriMstr0RQ     ,
    PeriMstr0AK     ,
    PeriMstr0RW     ,
    PeriMstr0BE     ,
    PeriMstr0ADR    ,
    PeriMstr0WD     ,
    PeriMstr0RD     ,

    //======= Peri Bus (Host1) ========
    PeriMstr1RQ     ,
    PeriMstr1AK     ,
    PeriMstr1RW     ,
    PeriMstr1BE     ,
    PeriMstr1ADR    ,
    PeriMstr1WD     ,
    PeriMstr1RD     ,

    //======= Peri Bus (Host2) ========
    PeriMstr2RQ     ,
    PeriMstr2AK     ,
    PeriMstr2RW     ,
    PeriMstr2BE     ,
    PeriMstr2ADR    ,
    PeriMstr2WD     ,
    PeriMstr2RD     ,

    //======= VGA ========================
    VgaCRQ      ,
    VgaCAK      ,
    VgaADR      ,
    VgaRD       ,
    VgaRDAK     ,

    VS              ,
    HS              ,
    VR              ,
    VG              ,
    VB              ,

    //======= IRQ ========================
    IrqOut0         ,
    IrqOut1         ,

    //======= Gpio ========================
    SlideSW         ,
    PushSW          ,
    RotCenter       ,
    RotA            ,
    RotB            ,

    LcdRW           ,
    LcdE            ,
    LcdRS           ,
    LcdDB           ,
    LED             ,
    GpInOutMode     ,
    GpInPut         ,
    GpOutPut        ,

    //======= FifoUart =====================
    RxD             ,
    TxD             ,

    //======= PMon0  =====================
    P0CoreDbgRq     ,
    P0CoreGoAhead   ,
    P0Bubble        ,
    P0CoreInstRQ    ,
    P0CoreInstAK    ,
    P0CoreInstHit   ,
    P0CoreDataRQ    ,
    P0CoreDataAK    ,
    P0CoreDataHit   ,

    //======= PMon1  =====================
    P1CoreDbgRq     ,
    P1CoreGoAhead   ,
    P1Bubble        ,
    P1CoreInstRQ    ,
    P1CoreInstAK    ,
    P1CoreInstHit   ,
    P1CoreDataRQ    ,
    P1CoreDataAK    ,
    P1CoreDataHit
);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           Clk             ; // 70MHz
    input           VgaClk          ; // 50MHz
    input           RstN            ;

    input           Clk16x115k2     ; // for uart 115200x16Hz
    input           Clk16x56k       ; // for uart  56000x16Hz
    input           Clk16x38k4      ; // for uart  38400x16Hz
    input           Clk16x19k2      ; // for uart  19200x16Hz
    input           Clk16x9k6       ; // for uart   9600x16Hz

    //======= Peri Bus (Host0) ========
    input           PeriMstr0RQ     ;
    output          PeriMstr0AK     ;
    input           PeriMstr0RW     ;
    input   [3:0]   PeriMstr0BE     ;
    input   [15:0]  PeriMstr0ADR    ;
    input   [31:0]  PeriMstr0WD     ;
    output  [31:0]  PeriMstr0RD     ;

    //======= Peri Bus (Host1) ========
    input           PeriMstr1RQ     ;
    output          PeriMstr1AK     ;
    input           PeriMstr1RW     ;
    input   [3:0]   PeriMstr1BE     ;
    input   [15:0]  PeriMstr1ADR    ;
    input   [31:0]  PeriMstr1WD     ;
    output  [31:0]  PeriMstr1RD     ;

    //======= Peri Bus (Host2) ========
    input           PeriMstr2RQ     ;
    output          PeriMstr2AK     ;
    input           PeriMstr2RW     ;
    input   [3:0]   PeriMstr2BE     ;
    input   [15:0]  PeriMstr2ADR    ;
    input   [31:0]  PeriMstr2WD     ;
    output  [31:0]  PeriMstr2RD     ;

    //======= VGA ========================
    //======= MEMORY ============
    output          VgaCRQ          ;
    input           VgaCAK          ;
    output  [25:0]  VgaADR          ;
    input   [31:0]  VgaRD           ;
    input           VgaRDAK         ;

    //======= RGB ============
    output          VS              ;
    output          HS              ;
    output          VR              ;
    output          VG              ;
    output          VB              ;

    //======= IRQ ========================
    output          IrqOut0         ;
    output          IrqOut1         ;

    //======= Gpio ========================
    input   [3:0]   SlideSW     ;
    input   [3:0]   PushSW      ;  //[3]:NOTH, [2]:SOUTH, [1]:EAST, [0]:WEST
    input           RotCenter   ;
    input           RotA        ;
    input           RotB        ;

    output          LcdRW       ;
    output          LcdE        ;
    output          LcdRS       ;
    output  [11:8]  LcdDB       ; // Not Connect
    output  [7:0]   LED         ;
    output  [11:0]  GpInOutMode ;
    input   [11:0]  GpInPut     ;
    output  [11:0]  GpOutPut    ;

    //======= FifoUart =====================
    input           RxD         ;   // UART RxD
    output          TxD         ;   // UART TxD


    //======= PMon0  =====================
    input           P0CoreDbgRq       ;
    input           P0CoreGoAhead     ;
    input           P0Bubble          ;
    input           P0CoreInstRQ      ;
    input           P0CoreInstAK      ;
    input           P0CoreInstHit     ;
    input           P0CoreDataRQ      ;
    input           P0CoreDataAK      ;
    input           P0CoreDataHit     ;

    //======= PMon1  =====================
    input           P1CoreDbgRq       ;
    input           P1CoreGoAhead     ;
    input           P1Bubble          ;
    input           P1CoreInstRQ      ;
    input           P1CoreInstAK      ;
    input           P1CoreInstHit     ;
    input           P1CoreDataRQ      ;
    input           P1CoreDataAK      ;
    input           P1CoreDataHit     ;

    //==================================================
    //======== wire & reg ==============================
    //==================================================

    //======= Peri Bus (Slave) =======
    //======= Peri Bus (Slave) =======
    wire            Irq0CS          ;
    wire            Irq1CS          ;
    wire            Timer0CS        ;
    wire            Timer1CS        ;
    wire            UartCS          ;
    wire            GpioCS          ;
    wire            VgaCS           ;
    wire            PMon0CS         ;
    wire            PMon1CS         ;
    wire            OcmCS           ;

    wire            PeriSlaveRQ     ;
    wire            PeriSlaveRW     ;
    wire    [3:0]   PeriSlaveBE     ;
    wire    [11:0]  PeriSlaveADR    ;
    wire    [31:0]  PeriSlaveWD     ;

    wire    [31:0]  Irq0RD          ;
    wire    [31:0]  Irq1RD          ;
    wire    [31:0]  Timer0RD        ;
    wire    [31:0]  Timer1RD        ;
    wire    [31:0]  UartRD          ;
    wire    [31:0]  GpioRD          ;
    wire    [31:0]  VgaPeriRD       ;
    wire    [31:0]  PMon0RD         ;
    wire    [31:0]  PMon1RD         ;
    wire    [31:0]  OcmRD           ;

    wire            Timer0Irq       ;
    wire            Timer1Irq       ;
    wire            UaSendIrq       ;
    wire            UaRecvIrq       ;

    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    //==================================================
    //======== P3PeriBusArbt  ==========================
    //==================================================
    P3dPeriAbt PeriAbt(
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),

        .PeriMstr0RQ     ( PeriMstr0RQ     ),
        .PeriMstr0AK     ( PeriMstr0AK     ),
        .PeriMstr0RW     ( PeriMstr0RW     ),
        .PeriMstr0BE     ( PeriMstr0BE     ),
        .PeriMstr0ADR    ( PeriMstr0ADR    ),
        .PeriMstr0WD     ( PeriMstr0WD     ),
        .PeriMstr0RD     ( PeriMstr0RD     ),

        .PeriMstr1RQ     ( PeriMstr1RQ     ),
        .PeriMstr1AK     ( PeriMstr1AK     ),
        .PeriMstr1RW     ( PeriMstr1RW     ),
        .PeriMstr1BE     ( PeriMstr1BE     ),
        .PeriMstr1ADR    ( PeriMstr1ADR    ),
        .PeriMstr1WD     ( PeriMstr1WD     ),
        .PeriMstr1RD     ( PeriMstr1RD     ),

        .PeriMstr2RQ     ( PeriMstr2RQ     ),
        .PeriMstr2AK     ( PeriMstr2AK     ),
        .PeriMstr2RW     ( PeriMstr2RW     ),
        .PeriMstr2BE     ( PeriMstr2BE     ),
        .PeriMstr2ADR    ( PeriMstr2ADR    ),
        .PeriMstr2WD     ( PeriMstr2WD     ),
        .PeriMstr2RD     ( PeriMstr2RD     ),

        .Irq0CS          ( Irq0CS          ),
        .Irq1CS          ( Irq1CS          ),
        .Timer0CS        ( Timer0CS        ),
        .Timer1CS        ( Timer1CS        ),
        .UartCS          ( UartCS          ),
        .GpioCS          ( GpioCS          ),
        .VgaCS           ( VgaCS           ),
        .PMon0CS         ( PMon0CS         ),
        .PMon1CS         ( PMon1CS         ),
        .OcmCS           ( OcmCS           ),

        .PeriSlaveRQ     ( PeriSlaveRQ     ),
        .PeriSlaveRW     ( PeriSlaveRW     ),
        .PeriSlaveBE     ( PeriSlaveBE     ),
        .PeriSlaveADR    ( PeriSlaveADR    ),
        .PeriSlaveWD     ( PeriSlaveWD     ),

        .Irq0RD          ( Irq0RD          ),
        .Irq1RD          ( Irq1RD          ),
        .Timer0RD        ( Timer0RD        ),
        .Timer1RD        ( Timer1RD        ),
        .UartRD          ( UartRD          ),
        .GpioRD          ( GpioRD          ),
        .VgaRD           ( VgaPeriRD       ),
        .PMon0RD         ( PMon0RD         ),
        .PMon1RD         ( PMon1RD         ),
        .OcmRD           ( OcmRD           )
    );

    //==================================================
    //======== Irq        ==============================
    //==================================================
    IrqCtrl IrqCtrl0(
        .Clk         ( Clk              ),
        .RstN        ( RstN             ),

        .HostCS      ( Irq0CS           ),
        .HostRQ      ( PeriSlaveRQ      ),
        .HostRW      ( PeriSlaveRW      ),
        .HostADR     ( PeriSlaveADR[3:0]),
        .HostWD      ( PeriSlaveWD      ),
        .HostRD      ( Irq0RD           ),

        .Timer0Irq   ( Timer0Irq        ),
        .Timer1Irq   ( Timer1Irq        ),

        .UaSendIrq   ( UaSendIrq        ),
        .UaRecvIrq   ( UaRecvIrq        ),

        .IrqOut      ( IrqOut0          )
    );

    IrqCtrl IrqCtrl1(
        .Clk         ( Clk              ),
        .RstN        ( RstN             ),

        .HostCS      ( Irq1CS           ),
        .HostRQ      ( PeriSlaveRQ      ),
        .HostRW      ( PeriSlaveRW      ),
        .HostADR     ( PeriSlaveADR[3:0]),
        .HostWD      ( PeriSlaveWD      ),
        .HostRD      ( Irq1RD           ),

        .Timer0Irq   ( Timer0Irq        ),
        .Timer1Irq   ( Timer1Irq        ),

        .UaSendIrq   ( UaSendIrq        ),
        .UaRecvIrq   ( UaRecvIrq        ),

        .IrqOut      ( IrqOut1          )
    );

    //==================================================
    //======== Timer      ==============================
    //==================================================
    Timer Timer0(
        .Clk         ( Clk              ),
        .RstN        ( RstN             ),

        .HostCS      ( Timer0CS         ),
        .HostRQ      ( PeriSlaveRQ      ),
        .HostRW      ( PeriSlaveRW      ),
        .HostADR     ( PeriSlaveADR[3:0]),
        .HostWD      ( PeriSlaveWD      ),
        .HostRD      ( Timer0RD         ),

        .TimerIrq    ( Timer0Irq        )
    );

    Timer Timer1(
        .Clk         ( Clk              ),
        .RstN        ( RstN             ),

        .HostCS      ( Timer1CS         ),
        .HostRQ      ( PeriSlaveRQ      ),
        .HostRW      ( PeriSlaveRW      ),
        .HostADR     ( PeriSlaveADR[3:0]),
        .HostWD      ( PeriSlaveWD      ),
        .HostRD      ( Timer1RD         ),

        .TimerIrq    ( Timer1Irq        )
    );


    //==================================================
    //======== FifoUart   ==============================
    //==================================================
    FifoUart FifoUart(
        .Clk         ( Clk              ),
        .RstN        ( RstN             ),

        .Clk16x115k2 ( Clk16x115k2      ),
        .Clk16x56k   ( Clk16x56k        ),
        .Clk16x38k4  ( Clk16x38k4       ),
        .Clk16x19k2  ( Clk16x19k2       ),
        .Clk16x9k6   ( Clk16x9k6        ),

        .RxD         ( RxD              ),
        .TxD         ( TxD              ),

        .HostCS      ( UartCS           ),
        .HostRQ      ( PeriSlaveRQ      ),
        .HostRW      ( PeriSlaveRW      ),
        .HostADR     ( PeriSlaveADR[3:0]),
        .HostWD      ( PeriSlaveWD      ),
        .HostRD      ( UartRD           ),

        .UaSendIrq   ( UaSendIrq        ),
        .UaRecvIrq   ( UaRecvIrq        )
    );

    //==================================================
    //======== Gpio       ==============================
    //==================================================
    Gpio Gpio(
        .Clk         ( Clk              ),
        .RstN        ( RstN             ),

        .HostCS      ( GpioCS           ),
        .HostRQ      ( PeriSlaveRQ      ),
        .HostRW      ( PeriSlaveRW      ),
        .HostADR     ( PeriSlaveADR[3:0]),
        .HostWD      ( PeriSlaveWD      ),
        .HostRD      ( GpioRD           ),

        .SlideSW     ( SlideSW          ),
        .PushSW      ( PushSW           ),
        .RotCenter   ( RotCenter        ),
        .RotA        ( RotA             ),
        .RotB        ( RotB             ),

        .LcdRW       ( LcdRW            ),
        .LcdE        ( LcdE             ),
        .LcdRS       ( LcdRS            ),
        .LcdDB       ( LcdDB            ),
        .LED         ( LED              ),
        .GpInOutMode ( GpInOutMode      ),
        .GpInPut     ( GpInPut          ),
        .GpOutPut    ( GpOutPut         )
    );

    //==================================================
    //======== VGA        ==============================
    //==================================================
    VgaTop VgaTop(
        .Clk         ( Clk              ),
        .VgaClk      ( VgaClk           ),
        .RstN        ( RstN             ),

        .HostCS      ( VgaCS            ),
        .HostRQ      ( PeriSlaveRQ      ),
        .HostRW      ( PeriSlaveRW      ),
        .HostADR     ( PeriSlaveADR[3:0]),
        .HostWD      ( PeriSlaveWD      ),
        .HostRD      ( VgaPeriRD        ),

        .VgaCRQ      ( VgaCRQ           ),
        .VgaCAK      ( VgaCAK           ),
        .VgaADR      ( VgaADR           ),
        .VgaRD       ( VgaRD            ),
        .VgaRDAK     ( VgaRDAK          ),

        .VS          ( VS               ),
        .HS          ( HS               ),
        .VR          ( VR               ),
        .VG          ( VG               ),
        .VB          ( VB               )
    );

    //==================================================
    //======== PMon       ==============================
    //==================================================
    PMon PMon0(
        .Clk         ( Clk              ),
        .RstN        ( RstN             ),

        .HostCS      ( PMon0CS          ),
        .HostRQ      ( PeriSlaveRQ      ),
        .HostRW      ( PeriSlaveRW      ),
        .HostADR     ( PeriSlaveADR[3:0]),
        .HostWD      ( PeriSlaveWD      ),
        .HostRD      ( PMon0RD          ),

        .DbgRq       ( P0CoreDbgRq      ),
        .GoAhead     ( P0CoreGoAhead    ),
        .Bubble      ( P0Bubble         ),
        .InstRQ      ( P0CoreInstRQ     ),
        .InstAK      ( P0CoreInstAK     ),
        .InstHit     ( P0CoreInstHit    ),
        .DataRQ      ( P0CoreDataRQ     ),
        .DataAK      ( P0CoreDataAK     ),
        .DataHit     ( P0CoreDataHit    )
    );

    //==================================================
    //======== PMon       ==============================
    //==================================================
    PMon PMon1(
        .Clk         ( Clk              ),
        .RstN        ( RstN             ),

        .HostCS      ( PMon1CS          ),
        .HostRQ      ( PeriSlaveRQ      ),
        .HostRW      ( PeriSlaveRW      ),
        .HostADR     ( PeriSlaveADR[3:0]),
        .HostWD      ( PeriSlaveWD      ),
        .HostRD      ( PMon1RD          ),

        .DbgRq       ( P1CoreDbgRq      ),
        .GoAhead     ( P1CoreGoAhead    ),
        .Bubble      ( P1Bubble         ),
        .InstRQ      ( P1CoreInstRQ     ),
        .InstAK      ( P1CoreInstAK     ),
        .InstHit     ( P1CoreInstHit    ),
        .DataRQ      ( P1CoreDataRQ     ),
        .DataAK      ( P1CoreDataAK     ),
        .DataHit     ( P1CoreDataHit    )
    );

    //==================================================
    //======== OCM       ==============================
    //==================================================
    Ocm Ocm(
        .Clk         ( Clk              ),
        .RstN        ( RstN             ),

        .HostCS      ( OcmCS            ),
        .HostRQ      ( PeriSlaveRQ      ),
        .HostRW      ( PeriSlaveRW      ),
        .HostBE      ( PeriSlaveBE      ),
        .HostADR     ( PeriSlaveADR     ),
        .HostWD      ( PeriSlaveWD      ),
        .HostRD      ( OcmRD            )
);

endmodule
