//============================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : Performance Monitor
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//============================================================
`define     PMON_CNTR       2'h0
`define     PMON_COUNTER0   2'h1
`define     PMON_COUNTER1   2'h2

module PMon(
    //======== SYSTEM ==============
    Clk         ,
    RstN        ,

    //======== Peripheral Bus ======
    HostCS      ,
    HostRQ      ,
    HostRW      ,
    HostADR     ,
    HostWD      ,
    HostRD      ,

    //======== Input Signal ======
    DbgRq       ,
    GoAhead     ,
    Bubble      ,
    InstRQ      ,
    InstAK      ,
    InstHit     ,
    DataRQ      ,
    DataAK      ,
    DataHit

);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           Clk         ;
    input           RstN        ;

    //======== Peripheral Bus ======
    input           HostCS      ;
    input           HostRQ      ;
    input           HostRW      ;
    input   [3:0]   HostADR     ;
    input   [31:0]  HostWD      ;
    output  [31:0]  HostRD      ;

    //======== Input Signal ======
    input           DbgRq       ;
    input           GoAhead     ;
    input           Bubble      ;
    input           InstRQ      ;
    input           InstAK      ;
    input           InstHit     ;
    input           DataRQ      ;
    input           DataAK      ;
    input           DataHit     ;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    reg     [2:0]   Mode        ; // 000:CPI, 001:ICash, 010:DCash, 011:InstBus, 1xx:DataBus
    reg             Clear       ;
    reg             Run         ;

    reg             DbgRq1d     ;
    reg             GoAhead1d   ;
    reg             InstRQ1d    ;
    reg             InstAK1d    ;
    reg             InstHit1d   ;
    reg             DataRQ1d    ;
    reg             DataAK1d    ;
    reg             DataHit1d   ;

    reg     [31:0]  Counter0    ;
    reg     [31:0]  Counter1    ;

    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    //======== ModeReg ===================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            Mode     <= 3'h0;
            Clear    <= 1'h0;
            Run      <= 1'h0;
        end
        else if(HostCS & HostRQ & HostRW & (HostADR[3:2] == `PMON_CNTR )) begin
            Mode     <= HostWD[6:4];
            Clear    <= HostWD[1];
            Run      <= HostWD[0];
        end
    end

    //======== ReadData ===================================
    assign  HostRD = (HostADR[3:2] == `PMON_CNTR    )? {25'h0, Mode, 2'h0, Clear, Run} :
                     (HostADR[3:2] == `PMON_COUNTER0)? Counter0                        :
                     (HostADR[3:2] == `PMON_COUNTER1)? Counter1                        : 32'h0;

    //======== Input Latch ===================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            DbgRq1d     <= 1'h0;
            GoAhead1d   <= 1'h0;
            InstRQ1d    <= 1'h0;
            InstAK1d    <= 1'h0;
            InstHit1d   <= 1'h0;
            DataRQ1d    <= 1'h0;
            DataAK1d    <= 1'h0;
            DataHit1d   <= 1'h0;
        end
        else begin
            DbgRq1d     <= DbgRq    ;
            GoAhead1d   <= GoAhead & ~Bubble ;
            InstRQ1d    <= InstRQ   ;
            InstAK1d    <= InstAK   ;
            InstHit1d   <= InstHit  ;
            DataRQ1d    <= DataRQ   ;
            DataAK1d    <= DataAK   ;
            DataHit1d   <= DataHit  ;
        end
    end

    //======== Counter0 ===================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            Counter0 <= 32'h0;
        end
        else if(Clear) begin
            Counter0 <= 32'h0;
        end
        else if(Run) begin
            if(Mode == 3'b000) begin
                if(~DbgRq1d)            Counter0 <= Counter0 + 1'b1;
            end
            else if((Mode == 3'b001) | (Mode == 3'b011)) begin
                if(InstRQ1d & InstAK1d) Counter0 <= Counter0 + 1'b1;
            end
            else if((Mode == 3'b010) | (Mode[2] == 1'b1)) begin
                if(DataRQ1d & DataAK1d) Counter0 <= Counter0 + 1'b1;
            end
        end
    end

    //======== Counter1 ===================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            Counter1 <= 32'h0;
        end
        else if(Clear) begin
            Counter1 <= 32'h0;
        end
        else if(Run) begin
            if(Mode == 3'b000) begin
                if(GoAhead1d)            Counter1 <= Counter1 + 1'b1;
            end
            else if(Mode == 3'b001) begin
                if(InstAK1d & InstHit1d) Counter1 <= Counter1 + 1'b1;
            end
            else if(Mode == 3'b010) begin
                if(DataAK1d & DataHit1d) Counter1 <= Counter1 + 1'b1;
            end
            else if(Mode == 3'b011) begin
                if(InstRQ1d & ~InstAK1d) Counter1 <= Counter1 + 1'b1;
            end
            else begin
                if(DataRQ1d & ~DataAK1d) Counter1 <= Counter1 + 1'b1;
            end
        end
    end

endmodule
