//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : *(PSM3 TARGET SYSTEM)
//  FUNCTION      : Psm3 Common Divisor
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//=======================================================================
`define     FUNC_MOV    4'h0
`define     FUNC_MOVL   4'h1
`define     FUNC_MOVH   4'h2
`define     FUNC_ADD    4'h3
`define     FUNC_SUB    4'h4
`define     FUNC_AND    4'h5
`define     FUNC_OR     4'h6
`define     FUNC_XOR    4'h7
`define     FUNC_LSR    4'h8
`define     FUNC_LSL    4'h9
`define     FUNC_ASR    4'hA
`define     FUNC_MUL    4'hB
`define     FUNC_DIVU   4'hC
`define     FUNC_DIV    4'hD
`define     FUNC_MODU   4'hE
`define     FUNC_MOD    4'hF

module P3ComDiv(
    //======== SYSTEM ===============
    Clk                 ,
    RstN                ,
    //======== P0 =================
    P0ComDivRQ          ,
    P0ComDivAK          ,
    P0ComDivCmpl        ,
    P0ComDivInB         ,
    P0ComDivInC         ,
    P0ComFunc           ,

    //======== P1 =================
    P1ComDivRQ          ,
    P1ComDivAK          ,
    P1ComDivCmpl        ,
    P1ComDivInB         ,
    P1ComDivInC         ,
    P1ComFunc           ,

    //======== Result ===============
    ComDivRes
);
    //==========================================================
    //======== port  ===========================================
    //==========================================================
    //======== SYSTEM ========================
    input           Clk                 ;
    input           RstN                ;

    //======== P0 =================
    input           P0ComDivRQ          ; // ComDiv REQ
    output          P0ComDivAK          ; // ComDiv ACK
    output          P0ComDivCmpl        ; // ComDiv Complete
    input   [31:0]  P0ComDivInB         ; // PortB Data
    input   [31:0]  P0ComDivInC         ; // PortC Data
    input   [3:0]   P0ComFunc           ; // FUNCTION

    //======== P1 =================
    input           P1ComDivRQ          ; // ComDiv REQ
    output          P1ComDivAK          ; // ComDiv ACK
    output          P1ComDivCmpl        ; // ComDiv Complete
    input   [31:0]  P1ComDivInB         ; // PortB Data
    input   [31:0]  P1ComDivInC         ; // PortC Data
    input   [3:0]   P1ComFunc           ; // FUNCTION

    //======== Result =================
    output  [31:0]  ComDivRes          ; // Result of ComDiv

    //==========================================================
    //======== wire & reg ======================================
    //==========================================================
    reg     [1:0]   ComDivRun          ; // ComDiv Run Flag
    reg     [5:0]   DivCounter         ; // ComDiv Counter
    reg     [31:0]  ComDivInB          ; // PortB Data
    reg     [31:0]  ComDivInC          ; // PortC Data

    wire    [31:0]  DivisorIn          ; // Divisor Input
    reg             DivisorMsb         ; // Dividend Input MSB
    reg     [31:0]  Divisor            ; // Divisor
    reg             DividendMsb        ; // Dividend Input MSB
    wire    [31:0]  Dividend           ; // Dividend Input

    reg     [31:0]  Quotient           ; // Quotient
    reg     [31:0]  Remainder          ; // Remainder

    wire    [31:0]  Quotient1          ; // Temporal Result of Quotient
    wire    [31:0]  Remainder1         ; // Temporal Result of Remainder
    wire    [32:0]  Remainder2         ; // Temporal Result of Remainder
    wire    [31:0]  Remainder3         ; // Temporal Result of Remainder

    reg     [3:0]   ComFuncIn          ; // FUNCTION input
    reg     [3:0]   ComFunc            ; // FUNCTION
    reg             P0ComDivAK         ; // ComDiv ACK
    reg             P0ComDivCmpl       ; // ComDiv Complete
    reg             P1ComDivAK         ; // ComDiv ACK
    reg             P1ComDivCmpl       ; // ComDiv Complete
    wire    [31:0]  ComDivRes0         ; // Result of ComDiv
    wire    [31:0]  ComDivRes1         ; // Result of ComDiv
    reg     [31:0]  ComDivRes          ; // Result of ComDiv

    //==========================================================
    //======== BODY       ======================================
    //==========================================================
    //=== P0&P1 ComDivAK and Input Data Selecter ===============
    always @(ComDivRun or P0ComDivRQ or P1ComDivRQ or
             P0ComDivInB or P0ComDivInC or P0ComFunc or
             P1ComDivInB or P1ComDivInC or P1ComFunc) begin
        P0ComDivAK = 1'b0;
        P1ComDivAK = 1'b0;
        ComDivInB  = P0ComDivInB;
        ComDivInC  = P0ComDivInC;
        ComFuncIn  = P0ComFunc;
        if(ComDivRun == 2'b00) begin
            if(P0ComDivRQ) begin
                P0ComDivAK = 1'b1;
                P1ComDivAK = 1'b0;
            end
            else if(P1ComDivRQ) begin
                P0ComDivAK = 1'b0;
                P1ComDivAK = 1'b1;
                ComDivInB  = P1ComDivInB;
                ComDivInC  = P1ComDivInC;
                ComFuncIn  = P1ComFunc;
            end
        end
    end

    //======== ComDivRun ========================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            ComDivRun <= 2'b00;
        end
        else if(ComDivRun == 2'b00) begin
            ComDivRun <= {P1ComDivAK, P0ComDivAK};
        end
        else if(DivCounter[5]) begin
            ComDivRun <= 2'b00;
        end
    end

    //======== DivCounter ========================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            DivCounter <= 6'h0;
        end
        else if((ComDivRun == 2'b00) | DivCounter[5])begin
            DivCounter <= 6'h0;
        end
        else begin
            DivCounter <= DivCounter + 1;
        end
    end

    //======== Main Operation ========================
    assign Dividend   = (((ComFuncIn == `FUNC_DIV) || (ComFuncIn == `FUNC_MOD)) & ComDivInB[31])? (~ComDivInB + 1'b1) : ComDivInB;
    assign DivisorIn  = (((ComFuncIn == `FUNC_DIV) || (ComFuncIn == `FUNC_MOD)) & ComDivInC[31])? (~ComDivInC + 1'b1) : ComDivInC;
    assign Quotient1  = {Quotient[30:0], ~Remainder2[32]};
    assign Remainder1 = {Remainder[30:0], Quotient[31]};
    assign Remainder2 = Remainder1 - Divisor;
    assign Remainder3 = (Remainder2[32])? Remainder1 : Remainder2;
    assign ComDivRes0 = ((ComFunc == `FUNC_DIV) || (ComFunc == `FUNC_DIVU))? Quotient : Remainder;
    assign ComDivRes1 = (((ComFunc == `FUNC_DIV) & (DividendMsb != DivisorMsb)) |
                         ((ComFunc == `FUNC_MOD) &  DividendMsb               ))? ~ComDivRes0 + 1'b1 : ComDivRes0;
    always @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            ComFunc     <= 4'h0;
            Divisor     <= 32'h0;
            DividendMsb <= 1'b0;
            DivisorMsb  <= 1'b0;
            Quotient    <= 32'h0;
            Remainder   <= 32'h0;
            P1ComDivCmpl<= 1'b0;
            P0ComDivCmpl<= 1'b0;
            ComDivRes   <= 32'h0;
        end
        else if(ComDivRun == 2'b00) begin
            ComFunc     <= ComFuncIn;
            Divisor     <= DivisorIn;
            DividendMsb <= ComDivInB[31];
            DivisorMsb  <= ComDivInC[31];
            Quotient    <= Dividend;
            Remainder   <= 32'h0;
            P1ComDivCmpl<= 1'b0;
            P0ComDivCmpl<= 1'b0;
            ComDivRes   <= ComDivRes1;
        end
        else begin
            Quotient    <= Quotient1;
            Remainder   <= Remainder3;
            ComDivRes   <= ComDivRes1;
            if(DivCounter[5]) begin
                P1ComDivCmpl<= ComDivRun[1];
                P0ComDivCmpl<= ComDivRun[0];
            end
        end
    end

endmodule
