//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : *(PSM3 TARGET SYSTEM)
//  FUNCTION      : Psm3 Instruction Decoder
//
//                      Copyright (C) 2008-2010 by Tomoyuki Inomoto
//=======================================================================
// 20100327:UpDateAfter and PtrUpDate0 are removed
//         :Some signals about forwarding are added. (RegAFwdEna, RegBFwdEna, RegCFwdEna)
//=======================================================================

`define     FUNC_MOV        4'h0
`define     FUNC_MOVL       4'h1
`define     FUNC_MOVH       4'h2
`define     FUNC_ADD        4'h3
`define     FUNC_SUB        4'h4
`define     FUNC_AND        4'h5
`define     FUNC_OR         4'h6
`define     FUNC_XOR        4'h7
`define     FUNC_LSR        4'h8
`define     FUNC_LSL        4'h9
`define     FUNC_ASR        4'hA
`define     FUNC_MUL        4'hB
`define     FUNC_DIVU       4'hC
`define     FUNC_DIV        4'hD
`define     FUNC_MODU       4'hE
`define     FUNC_MOD        4'hF

`define     LDW_STW         4'h0
`define     LDW_STW_UB      4'h1
`define     LDW_STW_UA      4'h2
`define     LDH_STH         4'h3
`define     LDH_STH_UB      4'h4
`define     LDH_STH_UA      4'h5
`define     LDB_STB         4'h6
`define     LDB_STB_UB      4'h7
`define     LDB_STB_UA      4'h8
`define     LDSH            4'h9
`define     LDSH_UB         4'hA
`define     LDSH_UA         4'hB
`define     LDSB            4'hC
`define     LDSB_UB         4'hD
`define     LDSB_UA         4'hE
`define     DRAINRQ         4'hF



`define     EQ  4'h1
`define     NE  4'h2
`define     HS  4'h3
`define     LO  4'h4
`define     MI  4'h5
`define     PL  4'h6
`define     VS  4'h7
`define     VC  4'h8
`define     HI  4'h9
`define     LS  4'hA
`define     GE  4'hB
`define     LT  4'hC
`define     GT  4'hD
`define     LE  4'hE
`define     AL  4'h0


module P3Id(
    //======== SYSTEM =================
    Clk             ,
    RstN            ,

    //======== input =================
    GoAhead         ,
    DecInstAK       ,
    IR              ,
    BrPredict       , // 20100403 add
    SR              ,
    
    UpDateCV        , // 20100327 add
    UpDateSZ        , // 20100327 add
    FwdOver         , // 20100327 add
    FwdCarry        , // 20100327 add
    FwdSign         , // 20100327 add
    FwdZero         , // 20100327 add

    IEforID         ,

    //========= IRQ ==================
    IrqIn           ,

    //======== output ================
    NextPcSel       ,
    PcWrited        ,

    RfWrAdr0        ,
    RfWe0           ,

    JmpAdrMode      ,

    RfRdAdrA        ,
    RfRdAdrB        ,
    RfRdAdrC        ,

    ImmSel          ,

    Func0           ,
    AluAct0         ,

    DataRQ0         ,
    DataRW0         ,
    DataSIZE0       ,
    //DrainRQ0        , // 20100402 remove

    //UpDateAfter     , // 20100327
    //PtrUpDate0      , // 20100327

    EscapeSR        ,
    ReturnSR        ,

    SetIE           ,
    ClrIE           ,

    SetICE          ,
    ClrICE          ,

    SetDCE          ,
    ClrDCE          ,

    SetBL           ,
    ClrBL           ,

    Bubble          
    
    //RegAFwdEna      , // Reg A Forwarding Enable // 20100327 add
    //RegBFwdEna      , // Reg B Forwarding Enable // 20100327 add
    //RegCFwdEna        // Reg C Forwarding Enable // 20100327 add
);
    //=======================================
    //======== port  ========================
    //=======================================
    //======== SYSTEM =================
    input           Clk             ;
    input           RstN            ;

    //======== input =================
    input           GoAhead         ; // Go Ahead Pipeline
    input           DecInstAK       ; // Instruction Reqest From DECU
    input   [31:0]  IR              ; // Instruction Register
    input           BrPredict       ; // Branch Prediction act // 20100403
    input   [3:0]   SR              ; //0:Z, 1:S, 2:C, 3:V

    input           UpDateCV        ; // UpDate Carry & Over // 20100327 add
    input           UpDateSZ        ; // UpDate Sign & Zero  // 20100327 add
    input           FwdOver         ; // Flag(forwarding)    // 20100327 add
    input           FwdCarry        ; // Flag(forwarding)    // 20100327 add
    input           FwdSign         ; // Flag(forwarding)    // 20100327 add
    input           FwdZero         ; // Flag(forwarding)    // 20100327 add

    input           IEforID         ; // IrqEnable for Instruction Decorder

    //========= IRQ ==================
    input           IrqIn           ;

    //======== output ================
    output  [1:0]   NextPcSel       ; // 00:IncPC, 01:JmpAdr, 1*:Vector
    input           PcWrited        ; // PC is OverWrited by Debuger

    output  [4:0]   RfWrAdr0        ; // Register file Write Address
    output          RfWe0           ; // Register file Write Enable

    output  [1:0]   JmpAdrMode      ; // 00:PC + Imm, 01:PC+REG, 10:REG

    output  [3:0]   RfRdAdrA        ; // Register file Read Address A
    output  [3:0]   RfRdAdrB        ; // Register file Read Address B
    output  [4:0]   RfRdAdrC        ; // Register file Read Address C

    output          ImmSel          ; // Immdiate Select

    output  [3:0]   Func0           ; // ALU FUNCTION
    output          AluAct0         ; // ALU ACTIVE

    output          DataRQ0         ; // DATA BUS REQEST (1:REQ)
    output          DataRW0         ; // DATA BUS READ/WRITE (1:WRITE)
    output  [2:0]   DataSIZE0       ; // DATA BUS DATA SIZE (000:WORD, 010:HALF, 011:Signed HALF, 100:BYTE, 101:Signed Half)
    //output          DrainRQ0        ; // Data Cash Drain RQ (for Write Back Cash)  // 20100402 remove

    //output          UpDateAfter     ; // Pointer UpDate After Access // 20100327
    //output          PtrUpDate0      ; // Pointer UpDate Flag // 20100327

    output          EscapeSR        ; // EscapeSR
    output          ReturnSR        ; // ReturnSR

    output          SetIE           ; // SET   IE@SR
    output          ClrIE           ; // CLEAR IE@SR

    output          SetICE          ; // SET   ICE@SR
    output          ClrICE          ; // CLEAR ICE@SR

    output          SetDCE          ; // SET   DCE@SR
    output          ClrDCE          ; // CLEAR DCE@SR

    output          SetBL           ; // SET   BL@SR
    output          ClrBL           ; // CLEAR BL@SR

    output          Bubble          ; // Bubble
    
    //output          RegAFwdEna      ; // Reg A Forwarding Enable // 20100327 add
    //output          RegBFwdEna      ; // Reg B Forwarding Enable // 20100327 add
    //output          RegCFwdEna      ; // Reg C Forwarding Enable // 20100327 add
    
    
    
    //=======================================
    //======== wire & reg ===================
    //=======================================
    wire            IntoInt         ;
    wire            IntoJmp         ;
    wire            IntoCall        ;
    wire            IntoRti         ;
    reg             IntoInt1d       ;
    reg             IntoJmp1d       ;
    reg             IntoCall1d      ;
    reg             IntoRti1d       ;
    //reg             HitCondition    ; // 20100403 remove
    reg             HitCondition0   ; // 20100403 add
    wire            HitCondition    ; // 20100403 add
    wire            IE              ;
    wire            Over            ;
    wire            Carry           ;
    wire            Sign            ;
    wire            Zero            ;
    reg     [1:0]   NextPcSel       ; // 00:IncPC, 01:JmpAdr, 1*:Vector
    reg     [1:0]   JmpAdrMode      ; // 00:PC + Imm, 01:PC+REG, 10:REG
    reg             RfWe0           ;
    reg             AluAct0         ;
    reg             DataRQ0         ;
    wire            SetIE0          ;
    wire            ClrIE0          ;
    wire            SetICE0         ;
    wire            ClrICE0         ;
    wire            SetDCE0         ;
    wire            ClrDCE0         ;
    wire            SetBL0          ;
    wire            ClrBL0          ;
    reg             SetIE           ;
    reg             ClrIE           ;
    reg             SetICE          ;
    reg             ClrICE          ;
    reg             SetDCE          ;
    reg             ClrDCE          ;
    reg             SetBL           ;
    reg             ClrBL           ;
    wire            NotIntoJmpOrInt ;
    reg     [2:0]   DataSIZE0       ;

    //=======================================
    //======== BODY       ===================
    //=======================================

    //==== HitCondition =====================
    assign  IE    = IEforID;
    assign  Over  = (UpDateCV)? FwdOver  : SR[3];
    assign  Carry = (UpDateCV)? FwdCarry : SR[2];
    assign  Sign  = (UpDateSZ)? FwdSign  : SR[1];
    assign  Zero  = (UpDateSZ)? FwdZero  : SR[0];
    //always @(IR or Over or Carry or Sign or Zero) begin  // 20100403 remove
    //    case(IR[27:24])
    //        `EQ     :   HitCondition =  Zero;                           // Zero
    //        `NE     :   HitCondition = ~Zero;                           // Not Zero
    //        `HS     :   HitCondition = ~Carry;                          // unsigned Higher or Same
    //        `LO     :   HitCondition =  Carry;                          // unsigned lower
    //        `MI     :   HitCondition =  Sign;                           // Minus
    //        `PL     :   HitCondition = ~Sign;                           // Plus
    //        `VS     :   HitCondition =  Over;                           // OverFlow
    //        `VC     :   HitCondition = ~Over;                           // Not OverFlow
    //        `HI     :   HitCondition = ~(Carry | Zero);                 // unsigned Higher
    //        `LS     :   HitCondition =  (Carry | Zero);                 // unsigned lower or Same
    //        `GE     :   HitCondition = ~(Sign  ^ Over);                 // signed Grater than or Equal
    //        `LT     :   HitCondition =  (Sign  ^ Over);                 // signed Less than
    //        `GT     :   HitCondition = ~((Sign ^ Over) | Zero);         // signed Grater than or Equal
    //        `LE     :   HitCondition =  ((Sign ^ Over) | Zero);         // signed Less than or Eqal
    //        default :   HitCondition = 1'b1;                            // Always
    //    endcase
    //end
    always @(IR or Over or Carry or Sign or Zero) begin  // 20100403 add
        case(IR[27:24])
            `EQ     :   HitCondition0 =  Zero;                           // Zero
            `NE     :   HitCondition0 = ~Zero;                           // Not Zero
            `HS     :   HitCondition0 = ~Carry;                          // unsigned Higher or Same
            `LO     :   HitCondition0 =  Carry;                          // unsigned lower
            `MI     :   HitCondition0 =  Sign;                           // Minus
            `PL     :   HitCondition0 = ~Sign;                           // Plus
            `VS     :   HitCondition0 =  Over;                           // OverFlow
            `VC     :   HitCondition0 = ~Over;                           // Not OverFlow
            `HI     :   HitCondition0 = ~(Carry | Zero);                 // unsigned Higher
            `LS     :   HitCondition0 =  (Carry | Zero);                 // unsigned lower or Same
            `GE     :   HitCondition0 = ~(Sign  ^ Over);                 // signed Grater than or Equal
            `LT     :   HitCondition0 =  (Sign  ^ Over);                 // signed Less than
            `GT     :   HitCondition0 = ~((Sign ^ Over) | Zero);         // signed Grater than or Equal
            `LE     :   HitCondition0 =  ((Sign ^ Over) | Zero);         // signed Less than or Eqal
            default :   HitCondition0 = 1'b1;                            // Always
        endcase
    end
    assign HitCondition = (BrPredict)? ~HitCondition0 : HitCondition0;  // 20100403 add
    
    //======== IntoInt ====================================
    assign  IntoInt = (IrqIn & IE & ~ClrIE0 & ~IntoJmp & ~IntoJmp1d & ~IntoInt1d);
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) IntoInt1d <= 1'b0;
        //else if(GoAhead) begin
        else if(DecInstAK) begin
            if(IntoInt1d) IntoInt1d <= 1'b0;
            else          IntoInt1d <= IntoInt;
        end
        else if(PcWrited) begin
            IntoInt1d <= 1'b0;
        end
    end

    //======== IntoJmp ====================================
    //assign  NotIntoJmpOrInt = ({IntoInt, IntoInt1d, IntoJmp1d} == 3'b000);
    assign  NotIntoJmpOrInt = ({IntoInt1d, IntoJmp1d} == 2'b00);
    assign  IntoJmp = (~IR[31] & HitCondition & NotIntoJmpOrInt);
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) IntoJmp1d <= 1'b0;
        //else if(GoAhead) begin
        else if(DecInstAK) begin
            if(IntoJmp1d) IntoJmp1d <= 1'b0;
            else          IntoJmp1d <= IntoJmp;
        end
        else if(PcWrited) begin
            IntoJmp1d <= 1'b0;
        end
    end
    //======== IntoCall ====================================
    assign  IntoCall = ((IR[31:29] == 3'b001) & HitCondition & NotIntoJmpOrInt);
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) IntoCall1d <= 1'b0;
        //else if(GoAhead) begin
        else if(DecInstAK) begin
            if(IntoCall1d) IntoCall1d <= 1'b0;
            else           IntoCall1d <= IntoCall;
        end
        else if(PcWrited) begin
            IntoCall1d <= 1'b0;
        end
    end
    //======== IntoRti ====================================
    assign  IntoRti = ((IR[31:29] == 3'b010) & HitCondition & NotIntoJmpOrInt);
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) IntoRti1d <= 1'b0;
        //else if(GoAhead) begin
        else if(DecInstAK) begin
            if(IntoRti1d) IntoRti1d <= 1'b0;
            else          IntoRti1d <= IntoRti;
        end
        else if(PcWrited) begin
            IntoRti1d <= 1'b0;
        end
    end

    //=====[1:0] NextPcSel // 00:IncPC, 01:JmpAdr, 1*:Vector
    always  @(IntoInt or IntoJmp)
        if(IntoInt)         NextPcSel = 2'b10;
        else if(IntoJmp)    NextPcSel = 2'b01;
        else                NextPcSel = 2'b00;

    //=====[4:0] RfWrAdr0  // Register file Write Address
    assign  RfWrAdr0 =  (IntoInt   )? 5'hE :
                        (IntoCall1d)? 5'hD : {((IR[31:28]==4'b1100) & IR[15]), IR[23:20]};

    //=====      RfWe0     // Register file Write Enable
    always  @(IntoInt1d or IntoInt or IntoJmp1d or IntoCall1d or IR) begin
        if(IntoInt)         RfWe0 = 1'b1;
        else if(IntoInt1d)  RfWe0 = 1'b0;
        else if(IntoJmp1d) begin
            if(IntoCall1d)  RfWe0 = 1'b1;
            else            RfWe0 = 1'b0;
        end
        else if((IR[31:29] == 3'b100) | (IR[31:29] == 3'b110)) begin // LD & ALU
                            RfWe0 = 1'b1;
        end
        else                RfWe0 = 1'b0;
    end

    //=====[1:0] JmpAdrMode// 00:PC + Imm, 01:PC+REG, 10:REG
// 20090801 modify
//  always @(IR)
//      if     ((IR[31:28] == 4'b0000) | (IR[31:28] == 4'b0010)) JmpAdrMode = 2'b01;
//      else if((IR[31:28] == 4'b0110) | (IR[31:28] == 4'b0100)) JmpAdrMode = 2'b10;
//      else                                                     JmpAdrMode = 2'b00;
//
    always @(IR) begin
        if(IR[31:28] == 4'b0000) begin
            JmpAdrMode = 2'b01;
        end
        else if((IR[31:28] == 4'b0110) | (IR[31:28] == 4'b0100) | (IR[31:28] == 4'b0010)) begin
            JmpAdrMode = 2'b10;
        end
        else begin
            JmpAdrMode = 2'b00;
        end
    end
    //=====[3:0] RfRdAdrA  // Register file Read Address A
    assign  RfRdAdrA = IR[23:20];

    //=====[3:0] RfRdAdrB  // Register file Read Address B
    assign  RfRdAdrB = IR[19:16];

    //=====[4:0] RfRdAdrC  // Register file Read Address C
    assign  RfRdAdrC[3:0] = IR[ 3: 0];
    assign  RfRdAdrC[4]   = IR[ 4];

    //=====      ImmSel    // Immdiate Select
    assign  ImmSel = IR[28];

    //=====[3:0] Func0     // ALU FUNCTION
    assign  Func0 = (IR[31:30] == 2'b10)? `FUNC_ADD : IR[27:24];

    //=====      AluAct0   // ALU ACTIVE
    always  @(IntoInt1d or IntoInt or IntoJmp1d or IR) begin
        if(IntoInt1d | IntoInt)         AluAct0 = 1'b0;
        else if(IntoJmp1d)              AluAct0 = 1'b0;
        else if(IR[31:29] == 3'b110)    AluAct0 = 1'b1;
        else                            AluAct0 = 1'b0;
    end

    //=====      DataRQ0   // DATA BUS REQEST (1:REQ)
    always  @(IntoInt1d or IntoInt or IntoJmp1d or IR) begin
        if(IntoInt1d | IntoInt)         DataRQ0 = 1'b0;
        else if(IntoJmp1d)              DataRQ0 = 1'b0;
        else if(IR[31:30] == 2'b10)     DataRQ0 = 1'b1;
        else                            DataRQ0 = 1'b0;
    end

    //=====      DataRW0   // DATA BUS READ/WRITE (1:WRITE)
    assign  DataRW0 = DataRQ0 & (IR[31:29] == 3'b101) ;

    //=====[2:0] DataSIZE0 // DATA BUS DATA SIZE (000:WORD, 010:HALF, 011:Signed HALF, 100:BYTE, 101:Signed BYTE)
    always @(IR) begin
        case(IR[27:24])
            `LDW_STW        : DataSIZE0 = 3'b000;
            `LDW_STW_UB     : DataSIZE0 = 3'b000;
            `LDW_STW_UA     : DataSIZE0 = 3'b000;
            `LDH_STH        : DataSIZE0 = 3'b010;
            `LDH_STH_UB     : DataSIZE0 = 3'b010;
            `LDH_STH_UA     : DataSIZE0 = 3'b010;
            `LDB_STB        : DataSIZE0 = 3'b100;
            `LDB_STB_UB     : DataSIZE0 = 3'b100;
            `LDB_STB_UA     : DataSIZE0 = 3'b100;
            `LDSH           : DataSIZE0 = 3'b011;
            `LDSH_UB        : DataSIZE0 = 3'b011;
            `LDSH_UA        : DataSIZE0 = 3'b011;
            `LDSB           : DataSIZE0 = 3'b101;
            `LDSB_UB        : DataSIZE0 = 3'b101;
            `LDSB_UA        : DataSIZE0 = 3'b101;
            default         : DataSIZE0 = 3'b000;
        endcase
    end

    //===== DrainRQ0        ; // Data Cash Drain RQ (for Write Back Cash)
    //assign DrainRQ0 = (IR[29:24] == {2'b11, `DRAINRQ});  // 20100402 remove

    //=====  UpDateAfter  // Pointer UpDate After Access
    //assign  UpDateAfter =  ((IR[27:24] == `LDW_STW_UA) | // 20100327
    //                        (IR[27:24] == `LDH_STH_UA) |
    //                        (IR[27:24] == `LDB_STB_UA) |
    //                        (IR[27:24] == `LDSH_UA   ) |
    //                        (IR[27:24] == `LDSB_UA   ) );

    //=====  PtrUpDate0  // Pointer UpDate Flag
    //assign  PtrUpDate0  =  ((IR[27:24] == `LDW_STW_UB) | // 20100327
    //                        (IR[27:24] == `LDW_STW_UA) |
    //                        (IR[27:24] == `LDH_STH_UB) |
    //                        (IR[27:24] == `LDH_STH_UA) |
    //                        (IR[27:24] == `LDB_STB_UB) |
    //                        (IR[27:24] == `LDB_STB_UA) |
    //                        (IR[27:24] == `LDSH_UB   ) |
    //                        (IR[27:24] == `LDSH_UA   ) |
    //                        (IR[27:24] == `LDSB_UB   ) |
    //                        (IR[27:24] == `LDSB_UA   ) );

    //=====  EscapeSR  ===================
    assign  EscapeSR = IntoInt1d;

    //=====  ReturnSR  ===================
    assign  ReturnSR = IntoRti1d;

    //=====  SetIE   // SET   IE@SR
    assign  SetIE0 = (IR[31:24] == 8'b1110_1000); // EI
    always  @(IntoInt1d or IntoInt or IntoJmp1d or SetIE0) begin
        if(IntoInt1d | IntoInt)     SetIE = 1'b0;
        else if(IntoJmp1d)          SetIE = 1'b0;
        else if(SetIE0)             SetIE = 1'b1;
        else                        SetIE = 1'b0;
    end

    //=====  ClrIE   // CLEAR IE@SR
    assign  ClrIE0 = (IR[31:24] == 8'b1110_0000); // DI
    always  @(IntoInt1d or IntoInt or IntoJmp1d or ClrIE0) begin
        if(IntoInt)                 ClrIE = 1'b1;
        else if(IntoInt1d)          ClrIE = 1'b0;
        else if(IntoJmp1d)          ClrIE = 1'b0;
        else if(ClrIE0)             ClrIE = 1'b1;
        else                        ClrIE = 1'b0;
    end

    //=====  SetICE  // SET   ICE@SR
    assign  SetICE0 = (IR[31:24] == 8'b1110_1001); // EIC
    always  @(IntoInt1d or IntoInt or IntoJmp1d or SetICE0) begin
        if(IntoInt1d | IntoInt)     SetICE = 1'b0;
        else if(IntoJmp1d)          SetICE = 1'b0;
        else if(SetICE0)            SetICE = 1'b1;
        else                        SetICE = 1'b0;
    end

    //=====  ClrICE  // CLEAR ICE@SR
    assign  ClrICE0 = (IR[31:24] == 8'b1110_0001); // DIC
    always  @(IntoInt1d or IntoInt or IntoJmp1d or ClrICE0) begin
        if(IntoInt1d | IntoInt)     ClrICE = 1'b0;
        else if(IntoJmp1d)          ClrICE = 1'b0;
        else if(ClrICE0)            ClrICE = 1'b1;
        else                        ClrICE = 1'b0;
    end
    //=====  SetDCE  // SET   DCE@SR
    assign  SetDCE0 = (IR[31:24] == 8'b1110_1010); // EDC
    always  @(IntoInt1d or IntoInt or IntoJmp1d or SetDCE0) begin
        if(IntoInt1d | IntoInt)     SetDCE = 1'b0;
        else if(IntoJmp1d)          SetDCE = 1'b0;
        else if(SetDCE0)            SetDCE = 1'b1;
        else                        SetDCE = 1'b0;
    end
    //=====  ClrDCE  // CLEAR DCE@SR
    assign  ClrDCE0 = (IR[31:24] == 8'b1110_0010); // DDC
    always  @(IntoInt1d or IntoInt or IntoJmp1d or ClrDCE0) begin
        if(IntoInt1d | IntoInt)     ClrDCE = 1'b0;
        else if(IntoJmp1d)          ClrDCE = 1'b0;
        else if(ClrDCE0)            ClrDCE = 1'b1;
        else                        ClrDCE = 1'b0;
    end

    //=====  SetBL  // SET   BL@SR
    assign  SetBL0 = (IR[31:24] == 8'b1110_1011); // BLK
    always  @(IntoInt1d or IntoInt or IntoJmp1d or SetBL0) begin
        if(IntoInt1d | IntoInt)     SetBL = 1'b0;
        else if(IntoJmp1d)          SetBL = 1'b0;
        else if(SetBL0)             SetBL = 1'b1;
        else                        SetBL = 1'b0;
    end
    //=====  ClrBL  // CLEAR BL@SR
    assign  ClrBL0 = (IR[31:24] == 8'b1110_0011); // BULK
    always  @(IntoInt1d or IntoInt or IntoJmp1d or ClrBL0) begin
        if(IntoInt1d | IntoInt)     ClrBL = 1'b0;
        else if(IntoJmp1d)          ClrBL = 1'b0;
        else if(ClrBL0)             ClrBL = 1'b1;
        else                        ClrBL = 1'b0;
    end

    //==== Bubble =====================
    assign Bubble = IntoJmp | IntoInt;
    

endmodule
