//=======================================================================
//  TARGET PROJECT: *
//  TARGET SYSTEM : *
//  FUNCTION      : Debug Uart Top
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//=======================================================================
module DbgUart(
    Clk         ,
    RstN        ,
    UartCKx16   ,

    TxD         ,
    RxD         ,

    DbgWait     ,
    DbgWe       ,
    DbgRe       ,
    DbgAdr      ,
    DbgWD       ,
    DbgRD
);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           Clk         ;
    input           RstN        ;
    input           UartCKx16   ;   // Recv & Send Data Rate x 16

    //======== UART ==============
    output          TxD         ;
    input           RxD         ;

    //======== DEBUG BUS ==============
    input           DbgWait     ;
    output          DbgWe       ;
    output          DbgRe       ;
    output  [3:0]   DbgAdr      ;
    output  [31:0]  DbgWD       ;
    input   [31:0]  DbgRD       ;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    wire            RComp       ;//  Recv Complete
    wire            RDataRe     ;//  Recv Data Read Enable
    wire   [7:0]    RData       ;//  Recv Data
    wire            SEmpty      ;//  Send Buffer Empty
    wire            SDataWe     ;//  Send Data Write Enable
    wire   [7:0]    SData       ;//  Send Data

    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    //======== UART  ==========================
    Uart Uart(
        .Clk         ( Clk          ),
        .RstN        ( RstN         ),
        .UartCKx16   ( UartCKx16    ),
        .RxD         ( RxD          ),
        .TxD         ( TxD          ),
        .RComp       ( RComp        ),
        .RDataRe     ( RDataRe      ),
        .RData       ( RData        ),
        .SEmpty      ( SEmpty       ),
        .SDataWe     ( SDataWe      ),
        .SData       ( SData        )
    );

    //======== DEBUG BUS ACCESS CNTR  ==========================
    DbgCtrl DbgCtrl(
        .Clk         ( Clk          ),
        .RstN        ( RstN         ),
        .RComp       ( RComp        ),
        .RDataRe     ( RDataRe      ),
        .RData       ( RData        ),
        .SEmpty      ( SEmpty       ),
        .SDataWe     ( SDataWe      ),
        .SData       ( SData        ),
        .DbgWait     ( DbgWait      ),
        .DbgWe       ( DbgWe        ),
        .DbgRe       ( DbgRe        ),
        .DbgAdr      ( DbgAdr       ),
        .DbgWD       ( DbgWD        ),
        .DbgRD       ( DbgRD        )
    );

endmodule
