//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : Psm3 Dual-core On Chip Debugger
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//=======================================================================
`define     P3DBG_BUSIDLE   2'b00
`define     P3DBG_BUSRQ     2'b01
`define     P3DBG_BUSWAIT   2'b11
`define     P3DBG_BUSAK     2'b10

module P3dDbg(
    //======== SYSTEM ==============
    Clk         ,
    RstN        ,

    //======= to/from P3Core ================
    CashClr     ,
    P3dRstN     ,

    P0DbgRq     ,
    P0DbgRegRq  ,
    P0DbgEna    ,
    P0Break     ,
    P0DecInstAK ,
    P0DbgRWe    ,
    P0DbgRD     ,
    P0Bubble    ,

    DbgRAdr     ,
    DbgWD       ,

    P1DbgRq     ,
    P1DbgRegRq  ,
    P1DbgEna    ,
    P1Break     ,
    P1DecInstAK ,
    P1DbgRWe    ,
    P1DbgRD     ,
    P1Bubble    ,

    //======= to/form Internal Bus ================
    DbgBusCRQ   ,
    DbgBusCAK   ,
    DbgBusADR   ,
    DbgBusRW    ,
    DbgBusBE    ,
    DbgBusWD    ,
    DbgBusWDAK  ,
    DbgBusRD    ,
    DbgBusRDAK  ,

    //======= to/from DbgUart ================
    DbgUartWait ,
    DbgUartWe   ,
    DbgUartRe   ,
    DbgUartAdr  ,
    DbgUartWD   ,
    DbgUartRD
);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           Clk         ;
    input           RstN        ;

    //======= to/from PSM3dCore ================
    output          CashClr     ; // Clear Cash
    output          P3dRstN     ; // P0 & P1 Reset

    output          P0DbgRq     ; // P0 Debug Reqest
    output          P0DbgRegRq  ; // P0 Debug Reg Access Request
    input           P0DbgEna    ; // P0 Debug Enable
    input           P0Break     ; // P0 Break
    input           P0DecInstAK ; // P0 InsAK
    output          P0DbgRWe    ; // P0 Register Write Enable
    input   [31:0]  P0DbgRD     ; // P0 Register Read  Data
    input           P0Bubble    ; // P0 Bubble

    output  [4:0]   DbgRAdr     ; // Reg Address
    output  [31:0]  DbgWD       ; // Reg Write Data

    output          P1DbgRq     ; // P1 Debug Reqest
    output          P1DbgRegRq  ; // P1 Debug Reg Access Request
    input           P1DbgEna    ; // P1 Debug Enable
    input           P1Break     ; // P1 Break
    input           P1DecInstAK ; // P1 InsAK
    output          P1DbgRWe    ; // P1 Register Write Enable
    input   [31:0]  P1DbgRD     ; // P1 Register Read  Data
    input           P1Bubble    ; // P1 Bubble

    //======= to/form Internal Bus ================
    output          DbgBusCRQ   ; // Internal Bus Command Req
    input           DbgBusCAK   ; // Internal Bus Command Ack
    output  [27:0]  DbgBusADR   ; // Internal Bus Address
    output          DbgBusRW    ; // Internal Bus Read/Write 0:R, 1:W
    output  [3:0]   DbgBusBE    ; // Internal Bus Byte Enable
    output  [31:0]  DbgBusWD    ; // Internal Bus Write Data
    input           DbgBusWDAK  ; // Internal Bus Write Data Ack
    input   [31:0]  DbgBusRD    ; // Internal Bus Read Data
    input           DbgBusRDAK  ; // Internal Bus read Data Ack

    //======= to/from DbgUart ================
    output          DbgUartWait ; // DbgUart Wait
    input           DbgUartWe   ; // DbgUart Write Enable
    input           DbgUartRe   ; // DbgUart Read Enable
    input   [3:0]   DbgUartAdr  ; // DbgUart Address
    input   [31:0]  DbgUartWD   ; // DbgUart Write data
    output  [31:0]  DbgUartRD   ; // DbgUart Read Data

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    wire            BreakHit        ;
    reg             BreakMode       ;
    reg             AcsMode         ;
    reg             AdrInc          ;
    reg             ChkCodeClr      ;
    reg             CashClr         ;
    reg             P3dRstN         ;
    reg             PRst            ;
    reg             DbgPID          ;
    reg             P0StepGo        ;
    reg             P1StepGo        ;
    reg             P0DbgRq0        ;
    reg             P1DbgRq0        ;
    reg     [27:0]  DbgAdr          ;
    reg     [31:0]  ChkCode         ;
    reg             P0StepCyc       ;
    wire            NextP0StepCyc   ;
    reg             P1StepCyc       ;
    wire            NextP1StepCyc   ;
    wire            Core0Break      ;
    wire            Core1Break      ;
    reg             P0DecInstAK1d   ;
    reg             P1DecInstAK1d   ;
    wire            DataAdrHit      ;
    reg     [1:0]   DbgBusState     ;
    wire            DbgBusRQ0       ;
    reg     [31:0]  ContReg         ;
    reg     [31:0]  ReadData        ;
    reg     [31:0]  PReadData       ;
    reg     [31:0]  DbgUartRD       ;
    reg             P0BubbleReg     ;
    reg             P1BubbleReg     ;
    reg             P0DbgRegRq      ;
    reg             P1DbgRegRq      ;
    //====================================================================
    //======== BODY ======================================================
    //====================================================================

    //======================================================================
    //======= to/from P3Core ===============================================
    //======================================================================

    //======== output           P0DbgRq       =====================
    assign  Core0Break = (BreakMode == 1'b0)? (P0Break | P1Break) : P0Break;
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) P0DecInstAK1d <= 1'b0;
        else      P0DecInstAK1d <= P0DecInstAK;
    end
    assign  P0DbgRq     = (P0StepGo & P0StepCyc & ~P0DecInstAK1d)? 1'b0 : P0DbgRq0 | Core0Break;

    //======== output           P1DbgRq       =====================
    assign  Core1Break = (BreakMode == 1'b0)? (P0Break | P1Break) : P1Break;
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) P1DecInstAK1d <= 1'b0;
        else      P1DecInstAK1d <= P1DecInstAK;
    end
    assign  P1DbgRq     = (P1StepGo & P1StepCyc & ~P1DecInstAK1d)? 1'b0 : P1DbgRq0 | Core1Break;

    //======== wire         BreakHit     =====================
    assign  BreakHit    = Core0Break | Core1Break;

    //======== output   [3:0]   DbgRAdr      =====================
    assign  DbgRAdr     = DbgAdr[4:0];

    //======== output   [31:0]  DbgWD        =====================
    assign  DbgWD       = DbgUartWD;

    //======== output           DbgRWe       =====================
    assign  DataAdrHit  = (DbgUartAdr == 4'h2);
    assign  P0DbgRWe    = ~DbgPID & ~AcsMode & DbgUartWe & (DbgUartAdr == 4'h2);
    assign  P1DbgRWe    =  DbgPID & ~AcsMode & DbgUartWe & (DbgUartAdr == 4'h2);

    //======================================================================
    //======= to/form Internal Bus =========================================
    //======================================================================
    //======= DbgBusState =====================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            DbgBusState <= `P3DBG_BUSIDLE;
        end
        else if(DbgBusState == `P3DBG_BUSIDLE) begin
            if(DbgBusRQ0) DbgBusState <= `P3DBG_BUSRQ;
        end
        else if(DbgBusState == `P3DBG_BUSRQ) begin
            if(DbgBusWDAK | DbgBusRDAK) DbgBusState <= `P3DBG_BUSAK;
            else if(DbgBusCAK)          DbgBusState <= `P3DBG_BUSWAIT;
        end
        else if(DbgBusState == `P3DBG_BUSWAIT) begin
            if(DbgBusWDAK | DbgBusRDAK) DbgBusState <= `P3DBG_BUSAK;
        end
        else begin
            DbgBusState <= `P3DBG_BUSIDLE;
        end
    end

    //======== output           DbgBusRQ    =================
    assign  DbgBusRQ0  = AcsMode & (DbgUartWe | DbgUartRe) & DataAdrHit;
    assign  DbgBusCRQ  = (DbgBusState == `P3DBG_BUSRQ);

    //======== output   [27:2]  DbgBusADR    =================
    assign  DbgBusADR   = DbgAdr[27:0];
    //======== output           DbgBusRW     =================
    assign  DbgBusRW    = DbgUartWe;

    //======== output   [3:0]   DbgBusBE     =================
    assign  DbgBusBE    = 4'hF;

    //======== output   [31:0]  DbgBusWD     =================
    assign  DbgBusWD    = DbgUartWD;

    //======================================================================
    //======= to/from DbgUart      =========================================
    //======================================================================
    //======== output           DbgUartWait  =================
    assign  DbgUartWait = (DbgBusRQ0)?  (DbgBusState != `P3DBG_BUSAK) : 1'b0;

    //======== output  [31:0]   DbgUartRD    =================
    always  @(posedge Clk) P0BubbleReg <= P0Bubble;
    always  @(posedge Clk) P1BubbleReg <= P1Bubble;
    always  @(posedge Clk) begin
        ContReg    <= { BreakHit    ,
                        BreakMode   ,
                        AcsMode     ,
                        AdrInc      ,
                        ChkCodeClr  ,
                        CashClr     ,
                        PRst        ,
                        DbgPID      ,
                        P1BubbleReg ,
                        P0BubbleReg ,
                        P1Break     ,
                        P0Break     ,
                        2'h0        ,
                        P1StepGo    ,
                        P0StepGo    ,
                        6'h0        ,
                        P1DbgEna    ,
                        P0DbgEna    ,
                        2'h0        ,
                        P1DbgRegRq  ,
                        P0DbgRegRq  ,
                        2'h0        ,
                        P1DbgRq0    ,
                        P0DbgRq0    };
    end

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)               PReadData <= 32'h0;
        else if(DbgPID == 1'b0) PReadData <= P0DbgRD;
        else                    PReadData <= P1DbgRD;
    end
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            ReadData <= 32'h0;
        end
        else if(DbgBusRQ0) begin
            if(DbgBusRDAK) ReadData <= DbgBusRD;
        end
        else begin
            ReadData <= PReadData;
        end
    end

    always  @(DbgUartAdr or ContReg or DbgAdr or ReadData or ChkCode or P0StepCyc or P1StepCyc) begin
        case(DbgUartAdr)
            4'h0    : DbgUartRD <= ContReg;
            4'h1    : DbgUartRD <= {4'h0, DbgAdr};
            4'h2    : DbgUartRD <= ReadData;
            4'h3    : DbgUartRD <= ChkCode;
            4'h4    : DbgUartRD <= {30'h0, P0StepCyc};
            4'h5    : DbgUartRD <= {30'h0, P1StepCyc};
            default : DbgUartRD <= 32'h0;
        endcase
    end

    //======================================================================
    //======== CNTR REGS ===================================================
    //======================================================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            BreakMode   <= 1'b0;
            AcsMode     <= 1'b0;
            AdrInc      <= 1'b0;
            ChkCodeClr  <= 1'b0;
            CashClr     <= 1'b0;
            PRst        <= 1'b0;
            DbgPID      <= 1'b0;
            P1StepGo    <= 1'b0;
            P0StepGo    <= 1'b0;
            P1DbgRegRq  <= 1'b0;
            P0DbgRegRq  <= 1'b0;
            P1DbgRq0    <= 1'b1;
            P0DbgRq0    <= 1'b1;
        end
        else if(DbgUartWe & (DbgUartAdr == 4'h0)) begin
            BreakMode   <= DbgUartWD[30];
            AcsMode     <= DbgUartWD[29];
            AdrInc      <= DbgUartWD[28];
            ChkCodeClr  <= DbgUartWD[27];
            CashClr     <= DbgUartWD[26];
            PRst        <= DbgUartWD[25];
            DbgPID      <= DbgUartWD[24];
            P1StepGo    <= DbgUartWD[17];
            P0StepGo    <= DbgUartWD[16];
            P1DbgRegRq  <= DbgUartWD[5];
            P0DbgRegRq  <= DbgUartWD[4];
            P1DbgRq0    <= DbgUartWD[1];
            P0DbgRq0    <= DbgUartWD[0];
        end
        else begin
            PRst        <= 1'b0;
        end
    end

    //======== P3dRstN ======================================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            P3dRstN   <= 1'b0;
        end
        else begin
            P3dRstN   <= ~PRst;
        end
    end

    //======== DbgAdr ======================================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            DbgAdr  <= 28'h0;
        end
        else if(DbgUartWe & (DbgUartAdr == 4'h1)) begin
            DbgAdr  <= DbgUartWD[27:0];
        end
        else if(AdrInc & DbgBusRQ0 & ~DbgUartWait) begin
            DbgAdr  <= DbgAdr + 3'h4;
        end
    end

    //======== ChkCode ======================================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            ChkCode <= 32'h0;
        end
        else if(ChkCodeClr) begin
            ChkCode <= 32'h0;
        end
        else if(DataAdrHit & ~DbgUartWait) begin
            if     (DbgUartWe) ChkCode  <= ChkCode + DbgUartWD;
            else if(DbgUartRe) ChkCode  <= ChkCode + DbgUartRD;
        end
    end
    //======== P0StepCyc ======================================================
    assign NextP0StepCyc = P0StepCyc - 1'b1;
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            P0StepCyc   <= 1'h0;
        end
        else if(DbgUartWe & (DbgUartAdr == 4'h4)) begin
            P0StepCyc   <= DbgUartWD[0];
        end
        else if(P0StepGo & P0DecInstAK) begin
            if(P0StepCyc != 1'h0) P0StepCyc   <= NextP0StepCyc;
        end
    end

    //======== P1StepCyc ======================================================
    assign NextP1StepCyc = P1StepCyc - 1'b1;
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            P1StepCyc   <= 1'h0;
        end
        else if(DbgUartWe & (DbgUartAdr == 4'h5)) begin
            P1StepCyc   <= DbgUartWD[0];
        end
        else if(P1StepGo & P1DecInstAK) begin
            if(P1StepCyc != 1'h0) P1StepCyc   <= NextP1StepCyc;
        end
    end

endmodule
