//============================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : Psm3 Dual-core On Chip Debugger
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//=============================================================
module P3dOcd(
    //======== SYSTEM ==============
    Clk         ,
    RstN        ,
    UartCKx16   ,

    //======= to/from P3Core ================
    CashClr     ,
    P3dRstN     ,

    P0DbgRq     ,
    P0DbgRegRq  ,
    P0DbgEna    ,
    P0Break     ,
    P0DecInstAK ,
    P0DbgRWe    ,
    P0DbgRD     ,
    P0Bubble    ,

    DbgRAdr     ,
    DbgWD       ,

    P1DbgRq     ,
    P1DbgRegRq  ,
    P1DbgEna    ,
    P1Break     ,
    P1DecInstAK ,
    P1DbgRWe    ,
    P1DbgRD     ,
    P1Bubble    ,

    //======= to/form Internal Bus ================
    DbgBusCRQ   ,
    DbgBusCAK   ,
    DbgBusADR   ,
    DbgBusRW    ,
    DbgBusBE    ,
    DbgBusWD    ,
    DbgBusWDAK  ,
    DbgBusRD    ,
    DbgBusRDAK  ,

    //======= to/from DbgUart ================
    TxD         ,
    RxD
);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           Clk         ;
    input           RstN        ;
    input           UartCKx16   ;   // Recv & Send Data Rate x 16

    //======= to/from P2Core ================
    output          CashClr     ; // Clear Cash
    output          P3dRstN     ; // P0 & P1 Reset

    output          P0DbgRq     ; // P0 Debug Reqest
    output          P0DbgRegRq  ; // P0 Debug Reg Access Request
    input           P0DbgEna    ; // P0 Debug Enable
    input           P0Break     ; // P0 Break
    input           P0DecInstAK ; // P0 InsAK
    output          P0DbgRWe    ; // P0 Register Write Enable
    input   [31:0]  P0DbgRD     ; // P0 Register Read  Data
    input           P0Bubble    ; // P0 Bubble

    output  [4:0]   DbgRAdr     ; // Reg Address
    output  [31:0]  DbgWD       ; // Reg Write Data

    output          P1DbgRq     ; // P1 Debug Reqest
    output          P1DbgRegRq  ; // P1 Debug Reg Access Request
    input           P1DbgEna    ; // P1 Debug Enable
    input           P1Break     ; // P1 Break
    input           P1DecInstAK ; // P1 InsAK
    output          P1DbgRWe    ; // P1 Register Write Enable
    input   [31:0]  P1DbgRD     ; // P1 Register Read  Data
    input           P1Bubble    ; // P1 Bubble

    //======= to/form Internal Bus ================
    output          DbgBusCRQ   ; // Internal Bus Command Req
    input           DbgBusCAK   ; // Internal Bus Command Ack
    output  [27:0]  DbgBusADR   ; // Internal Bus Address
    output          DbgBusRW    ; // Internal Bus Read/Write 0:R, 1:W
    output  [3:0]   DbgBusBE    ; // Internal Bus Byte Enable
    output  [31:0]  DbgBusWD    ; // Internal Bus Write Data
    input           DbgBusWDAK  ; // Internal Bus Write Data Ack
    input   [31:0]  DbgBusRD    ; // Internal Bus Read Data
    input           DbgBusRDAK  ; // Internal Bus read Data Ack

    //======== UART ==============
    output          TxD         ;
    input           RxD         ;


    //==================================================
    //======== wire & reg ==============================
    //==================================================
    //======= to/from DbgUart ================
    wire            DbgUartWait ;
    wire            DbgUartWe   ;
    wire            DbgUartRe   ;
    wire    [3:0]   DbgUartAdr  ;
    wire    [31:0]  DbgUartWD   ;
    wire    [31:0]  DbgUartRD   ;
    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    P3dDbg P3dDbg(
        .Clk         ( Clk         ),
        .RstN        ( RstN        ),

        .CashClr     ( CashClr     ),
        .P3dRstN     ( P3dRstN     ),

        .P0DbgRq     ( P0DbgRq     ),
        .P0DbgRegRq  ( P0DbgRegRq  ),
        .P0DbgEna    ( P0DbgEna    ),
        .P0Break     ( P0Break     ),
        .P0DecInstAK ( P0DecInstAK ),
        .P0DbgRWe    ( P0DbgRWe    ),
        .P0DbgRD     ( P0DbgRD     ),
        .P0Bubble    ( P0Bubble    ),

        .DbgRAdr     ( DbgRAdr     ),
        .DbgWD       ( DbgWD       ),

        .P1DbgRq     ( P1DbgRq     ),
        .P1DbgRegRq  ( P1DbgRegRq  ),
        .P1DbgEna    ( P1DbgEna    ),
        .P1Break     ( P1Break     ),
        .P1DecInstAK ( P1DecInstAK ),
        .P1DbgRWe    ( P1DbgRWe    ),
        .P1DbgRD     ( P1DbgRD     ),
        .P1Bubble    ( P1Bubble    ),

        .DbgBusCRQ   ( DbgBusCRQ   ),
        .DbgBusCAK   ( DbgBusCAK   ),
        .DbgBusADR   ( DbgBusADR   ),
        .DbgBusRW    ( DbgBusRW    ),
        .DbgBusBE    ( DbgBusBE    ),
        .DbgBusWD    ( DbgBusWD    ),
        .DbgBusWDAK  ( DbgBusWDAK  ),
        .DbgBusRD    ( DbgBusRD    ),
        .DbgBusRDAK  ( DbgBusRDAK  ),

        .DbgUartWait ( DbgUartWait ),
        .DbgUartWe   ( DbgUartWe   ),
        .DbgUartRe   ( DbgUartRe   ),
        .DbgUartAdr  ( DbgUartAdr  ),
        .DbgUartWD   ( DbgUartWD   ),
        .DbgUartRD   ( DbgUartRD   )
    );

    DbgUart DbgUart(
        .Clk         ( Clk         ),
        .RstN        ( RstN        ),
        .UartCKx16   ( UartCKx16   ),

        .TxD         ( TxD         ),
        .RxD         ( RxD         ),

        .DbgWait     ( DbgUartWait ),
        .DbgWe       ( DbgUartWe   ),
        .DbgRe       ( DbgUartRe   ),
        .DbgAdr      ( DbgUartAdr  ),
        .DbgWD       ( DbgUartWD   ),
        .DbgRD       ( DbgUartRD   )
    );

endmodule
