//============================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : Vga(video out interface) Top Module
//  VERSION       : Ver0.60
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//============================================================
module VgaTop(
    //========== SYS ============
    Clk         ,
    VgaClk      ,
    RstN        ,

    //========== HOST ============
    HostCS      ,
    HostRW      ,
    HostRQ      ,
    HostADR     ,
    HostWD      ,
    HostRD      ,

    //========== MEMORY ============
    VgaCRQ      ,
    VgaCAK      ,
    VgaADR      ,
    VgaRD       ,
    VgaRDAK     ,

    //========== VOUT ============
    VS          ,
    HS          ,
    VR          ,
    VG          ,
    VB
);
    //==================================================
    //======== port  ===================================
    //==================================================
    //========== SYSTEM ============
    input           Clk     ;
    input           VgaClk  ; // 100MHz
    input           RstN    ;

    //========== HOST ============
    input           HostCS  ;
    input           HostRW  ;
    input           HostRQ  ;
    input   [3:0]   HostADR ;
    input   [31:0]  HostWD  ;
    output  [31:0]  HostRD  ;

    //========== MEMORY ============
    output          VgaCRQ  ;
    input           VgaCAK  ;
    output  [25:0]  VgaADR  ;
    input   [31:0]  VgaRD   ;
    input           VgaRDAK ;

    //========== VOUT ============
    output          VS      ;
    output          HS      ;
    output          VR      ;
    output          VG      ;
    output          VB      ;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    wire            VgaMode     ;
    wire            VgaEna      ;
    wire    [25:4]  VgaBase0    ;
    wire    [25:4]  VgaBase1    ;
    wire            VgaFifoFull ;
    wire            VgaFifoWe   ;
    wire            VgaFifoRe   ;
    wire    [31:0]  VgaFifoRd   ;
    wire    [31:0]  VgaFifoWD   ;
    reg     [1:0]   VgaOutRstN  ;

    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    //========== VgaCR ============
    VgaCR VgaCR(
        .Clk         ( Clk              ),
        .RstN        ( RstN             ),

        .HostCS      ( HostCS           ),
        .HostRW      ( HostRW           ),
        .HostRQ      ( HostRQ           ),
        .HostADR     ( HostADR          ),
        .HostWD      ( HostWD           ),
        .HostRD      ( HostRD           ),

        .VgaMode     ( VgaMode          ),
        .VgaEna      ( VgaEna           ),
        .VgaBase0    ( VgaBase0         ),
        .VgaBase1    ( VgaBase1         )
    );

    //========== VgaRC ============
    VgaRC VgaRC(
        .Clk         ( Clk              ),
        .RstN        ( RstN             ),

        .VgaMode     ( VgaMode          ),
        .VgaEna      ( VgaEna           ),
        .VgaBase0    ( VgaBase0         ),
        .VgaBase1    ( VgaBase1         ),

        .VgaCRQ      ( VgaCRQ           ),
        .VgaCAK      ( VgaCAK           ),
        .VgaADR      ( VgaADR           ),
        .VgaRD       ( VgaRD            ),
        .VgaRDAK     ( VgaRDAK          ),

        .VgaFifoFull ( VgaFifoFull      ),
        .VgaFifoWe   ( VgaFifoWe        ),
        .VgaFifoWD   ( VgaFifoWD        )
    );

    //========== Fifo32bx512w ============
    Fifo32bx512w Fifo32bx512w(
        .wr_clk     ( Clk               ),
        .din        ( VgaFifoWD         ),
        .wr_en      ( VgaFifoWe         ),
        .rd_clk     ( VgaClk            ),
        .rd_en      ( VgaFifoRe         ),
        .rst        ( ~VgaEna           ),
        .dout       ( VgaFifoRd         ),
        .empty      (                   ),
        .full       (                   ),
        .prog_full  ( VgaFifoFull       ) // Full Flag Threshold is 500
    );

    //========== VgaVO ============
    always @(posedge VgaClk or negedge RstN) begin
        if(~RstN) VgaOutRstN <= 2'h0;
        else      VgaOutRstN <= {VgaOutRstN[0], VgaEna};
    end
    VgaVO VgaVO(
        .Clk         ( VgaClk           ),
        .RstN        ( VgaOutRstN[1]    ),

        .VgaFifoFull ( VgaFifoFull      ),
        .VgaFifoRe   ( VgaFifoRe        ),
        .VgaFifoRd   ( VgaFifoRd        ),

        .VS          ( VS               ),
        .HS          ( HS               ),
        .VR          ( VR               ),
        .VG          ( VG               ),
        .VB          ( VB               )
    );
endmodule
