//============================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : Vga(video out interface) Video Output signal controler
//  VERSION       : Ver0.60
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//============================================================

`define     H_PIX   10'd800
`define     H_SYN   10'd64
`define     H_BP    10'd80
`define     H_DISP  10'd640

`define     V_LINE  10'd521
`define     V_SYN   10'd3
`define     V_BP    10'd28
`define     V_DISP  10'd480

module VgaVO(
    Clk         ,
    RstN        ,

    VgaFifoFull ,
    VgaFifoRe   ,
    VgaFifoRd   ,

    VS          ,
    HS          ,
    VR          ,
    VG          ,
    VB
);
    //==================================================
    //======== port  ===================================
    //==================================================
    //========== SYSTEM ============
    input           Clk;    // 100MHz
    input           RstN;

    //========== FIFO ============
    input           VgaFifoFull;
    output          VgaFifoRe;
    input   [31:0]  VgaFifoRd;

    //========== VOUT ============
    output          VS;
    output          HS;
    output          VR;
    output          VG;
    output          VB;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    reg             VS;
    reg             HS;
    reg             VR;
    reg             VG;
    reg             VB;

    reg             HDISP;
    reg             VDISP;

    reg             Run;
    //reg             Ena;
    wire            Ena;
    reg     [3:0]   EnaCounter;

    reg     [3:0]   RunMask;
    reg     [1:0]   SyncFifoFull;

    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)   RunMask <= 4'b0000;
        else        RunMask <= {RunMask[2:0], 1'b1};
    end

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)   SyncFifoFull <= 2'b00;
        else        SyncFifoFull <= {SyncFifoFull[0], VgaFifoFull};
    end

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)              Run <= 1'b0;
        else if(SyncFifoFull)  Run <= (RunMask == 4'b1111);
    end
/*
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)       Ena <= 1'b0;
        else if(Run)    Ena <= ~Ena;
    end
*/
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)       EnaCounter <= 4'b0001;
        else if(Run)    EnaCounter <= {EnaCounter[2:0], EnaCounter[3]};
    end
    assign Ena = EnaCounter[3];

    //============ HSYNC ======================================
    reg     [9:0]   HCounter;
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                   HCounter <= 10'd1;
        else if(Ena) begin
            if(HCounter == `H_PIX)  HCounter <= 10'd1;
            else                    HCounter <= HCounter + 1'b1;
        end
    end

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                       HS <= 1'd0;
        else if(Ena) begin
            if(HCounter == `H_PIX)      HS <= 1'd0;
            else if(HCounter == `H_SYN) HS <= 1'b1;
        end
    end

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                                       HDISP <= 1'd0;
        else if(Ena) begin
            if(HCounter == (`H_SYN + `H_BP + `H_DISP))  HDISP <= 1'd0;
            else if(HCounter == (`H_SYN + `H_BP))       HDISP <= 1'b1;
        end
    end

    //============ VSYNC ======================================
    wire    VEna;
    assign  VEna = Ena & (HCounter == `H_PIX);

    reg     [9:0]   VCounter;
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                   VCounter <= 10'd1;
        else if(VEna) begin
            if(VCounter == `V_LINE) VCounter <= 10'd1;
            else                    VCounter <= VCounter + 1'b1;
        end
    end

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                       VS <= 1'd0;
        else if(VEna) begin
            if(VCounter == `V_LINE)     VS <= 1'd0;
            else if(VCounter == `V_SYN) VS <= 1'b1;
        end
    end

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                       VDISP <= 1'd0;
        else if(VEna) begin
            if(VCounter == (`V_SYN + `V_BP + `V_DISP))  VDISP <= 1'd0;
            else if(VCounter == (`V_SYN + `V_BP))       VDISP <= 1'b1;
        end
    end

    //============ RGB ======================================
    reg     [2:0]   PixCounter;
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                       PixCounter <= 3'h0;
        else if(HDISP & VDISP & Ena)    PixCounter <= PixCounter + 1'b1;
    end

    //assign  VgaFifoRe = (HDISP & VDISP & ~Ena & (PixCounter == 3'h0));
    assign  VgaFifoRe = (HDISP & VDISP & EnaCounter[2] & (PixCounter == 3'h0));

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)   begin
            {VR,VG,VB} <= 3'h0;
        end
        else if(Ena) begin
            if(HDISP & VDISP) begin
                case(PixCounter)
                    3'h0    : {VR,VG,VB} <= VgaFifoRd[30:28];
                    3'h1    : {VR,VG,VB} <= VgaFifoRd[26:24];
                    3'h2    : {VR,VG,VB} <= VgaFifoRd[22:20];
                    3'h3    : {VR,VG,VB} <= VgaFifoRd[18:16];
                    3'h4    : {VR,VG,VB} <= VgaFifoRd[14:12];
                    3'h5    : {VR,VG,VB} <= VgaFifoRd[10: 8];
                    3'h6    : {VR,VG,VB} <= VgaFifoRd[ 6: 4];
                    default : {VR,VG,VB} <= VgaFifoRd[ 2: 0];
                endcase
            end
            else begin
                {VR,VG,VB} <= 3'h0;
            end
        end
    end

endmodule
