﻿//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  FUNCTION      : Debugger
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//=======================================================================
using System;
using System.Collections.Generic;
using System.Text;

namespace Psm3Debugger
{
    public class DbgCom
    {
        //================================================================================================
        //=　定数宣言
        //================================================================================================
        public const uint P3DBG_OK = 0;
        public const uint P3DBG_NG = 1;

        public const uint P0 = 0;
        public const uint P1 = 1;
        public const uint PROCESSOR_RUN = 0;
        public const uint PROCESSOR_STOP = 1;
        public const uint NOT_BREAK_STAT = 0;

        public const uint CntrReg = 0x0;
        public const uint AddrReg = 0x1;
        public const uint DataReg = 0x2;
        public const uint ChkSumReg = 0x3;
        public const uint P0StepCycReg = 0x4;
        public const uint P1StepCycReg = 0x5;

        public const uint BreakHitBit = 0x80000000;
        public const uint BreakModeBit = 0x40000000;
        public const uint AccessModeBit = 0x20000000;
        public const uint AddrIncMode = 0x10000000;
        public const uint ClrChkSumBit = 0x08000000;
        public const uint ClrCashBit = 0x04000000;
        public const uint PRstBit = 0x02000000;
        public const uint PID0 = 0x00000000;
        public const uint PID1 = 0x01000000;
        public const uint PIDBit = 0x01000000;
        public const uint P1BubbleBit = 0x00800000;
        public const uint P0BubbleBit = 0x00400000;
        public const uint P1BreakBit = 0x00200000;
        public const uint P0BreakBit = 0x00100000;
        public const uint P1StepGoBit = 0x00020000;
        public const uint P0StepGoBit = 0x00010000;
        public const uint P1DebugEnaBit = 0x00000200;
        public const uint P0DebugEnaBit = 0x00000100;
        public const uint P1RegRqBit = 0x00000020;
        public const uint P0RegRqBit = 0x00000010;
        public const uint P1DebugReqBit = 0x00000002;
        public const uint P0DebugReqBit = 0x00000001;

        private uint ComOpenFlag = 0;


        //================================================================================================
        //=　変数宣言
        //================================================================================================
        private System.IO.Ports.SerialPort UART;
        public String ComPortNo = "COM1";
        //================================================================================================
        //=　コンストラクタ
        //================================================================================================
        public DbgCom()
        {
            //this.UART = new System.IO.Ports.SerialPort(this.components);
            this.UART = new System.IO.Ports.SerialPort();
            //this.UART.BaudRate = 115200;
            this.UART.BaudRate = 230400;
            this.UART.ReadTimeout = 100;
            //this.UART.ReadTimeout = 0;
            this.UART.WriteTimeout = 100;
            this.UART.PortName = ComPortNo;
        }
        //================================================================================================
        //=　メソッド
        //================================================================================================
        //=　オープン ====================================================================================
        public void DbgOpen()
        {
            if (ComOpenFlag == 0)
            {
                this.UART.PortName = ComPortNo;
                UART.Open();
                ComOpenFlag = 1;
            }
            
        }
        //=　クローズ ====================================================================================
        public void DbgClose()
        {
            //UART.Close();
        }
        //=　共通関数[ビットセット/クリア] =================================
        private uint SetBit(uint Reg, uint Bit)
        {
            return Reg | Bit;
        }
        private uint ClrBit(uint Reg, uint Bit)
        {
            return Reg & ~Bit;
        }
        //=　共通関数[デバッガ書き込み] ====================================
        public uint DbgWrite(uint Adr, uint Data)
        {
            uint i;
            byte[] SendBuf = new byte[8];               // UART SEND BUFFER

            SendBuf[0] = 0x16;
            SendBuf[1] = (byte)((Adr & 0xF) | 0xF0);
            SendBuf[2] = (byte)((Data >> 24) & 0xFF);
            SendBuf[3] = (byte)((Data >> 16) & 0xFF);
            SendBuf[4] = (byte)((Data >> 8) & 0xFF);
            SendBuf[5] = (byte)((Data >> 0) & 0xFF);
            SendBuf[6] = SendBuf[1];
            for (i = 2; i <= 5; i++)
            {
                SendBuf[6] = (byte)(SendBuf[6] ^ SendBuf[i]);
            }
            UART.Write(SendBuf, 0, 7);
            return P3DBG_OK;
        }
        //=　共通関数[デバッガバースト書き込み] ====================================
        public uint DbgWriteBurst(uint Adr, ref uint[] Data, uint Num)
        {
            uint i,j;
            byte[] SendBuf = new byte[2048];               // UART SEND BUFFER

            for (j = 0; j < Num; j++)
            {
                SendBuf[(j * 7) + 0] = 0x16;
                SendBuf[(j * 7) + 1] = (byte)((Adr & 0xF) | 0xF0);
                SendBuf[(j * 7) + 2] = (byte)((Data[j] >> 24) & 0xFF);
                SendBuf[(j * 7) + 3] = (byte)((Data[j] >> 16) & 0xFF);
                SendBuf[(j * 7) + 4] = (byte)((Data[j] >> 8) & 0xFF);
                SendBuf[(j * 7) + 5] = (byte)((Data[j] >> 0) & 0xFF);
                SendBuf[(j * 7) + 6] = SendBuf[(j * 7) + 1];
                for (i = 2; i <= 5; i++)
                {
                    SendBuf[(j * 7) + 6] = (byte)(SendBuf[(j * 7) + 6] ^ SendBuf[(j * 7) + i]);
                }
            }
            UART.Write(SendBuf, 0, (7 * (int)Num));
            return P3DBG_OK;
        }
        //=　共通関数[デバッガ読み込み] ====================================
        public uint DbgRead(uint Adr, ref uint Data)
        {
            uint Ret;
            uint ErrCounter;

            ErrCounter = 0;
            while (true)
            {
                Ret = DbgReadBody(Adr, ref Data);
                if (Ret == P3DBG_OK) break;
                ++ErrCounter;
                if (ErrCounter > 10)
                {
                    Ret = P3DBG_NG;
                    break;
                }
            }
            return Ret;
        }
        private uint DbgReadBody(uint Adr, ref uint Data)
        {
            uint Tmp;
            uint i;
            int Ret;
            byte[] SendBuf = new byte[4];               // UART SEND BUFFER
            byte[] RecvBuf = new byte[8];               // UART RECV BUFFER


            UART.DiscardInBuffer();

            SendBuf[0] = 0x16;
            SendBuf[1] = (byte)(Adr & 0xF);
            SendBuf[2] = SendBuf[1];
            UART.Write(SendBuf, 0, 3);

            //System.Threading.Thread.Sleep(10);
            for (i = 0; i < 6; i++)
            {
                try
                {
                    Ret = UART.Read(RecvBuf, (int)i, 1);
                }
                catch (System.TimeoutException)
                {
                    return P3DBG_NG;

                }
                if (Ret == 0)
                {
                    return P3DBG_NG;
                }
            }
            Tmp = ((uint)RecvBuf[1] << 24) & 0xFF000000;
            Tmp |= ((uint)RecvBuf[2] << 16) & 0x00FF0000;
            Tmp |= ((uint)RecvBuf[3] << 8) & 0x0000FF00;
            Tmp |= ((uint)RecvBuf[4] << 0) & 0x000000FF;
            Data = Tmp;
            return P3DBG_OK;
        }
        //=　共通関数[デバッガバースト読み込み] ====================================
        public uint DbgReadBurst(uint Adr, ref uint[] Data, uint Num) // Numの最大値は256
        {
            uint Ret;
            uint ErrCounter;

            ErrCounter = 0;
            while (true)
            {
                Ret = DbgReadBurstBody(Adr, ref Data, Num);
                if (Ret == P3DBG_OK) break;
                ++ErrCounter;
                if (ErrCounter > 10)
                {
                    Ret = P3DBG_NG;
                    break;
                }
            }
            return P3DBG_OK;
        }
        private uint DbgReadBurstBody(uint Adr, ref uint[] Data, uint Num)
        {
            uint Tmp;
            uint i;
            uint j;
            uint Rest;
            int Ret;
            byte[] SendBuf = new byte[4];               // UART SEND BUFFER
            byte[] RecvBuf = new byte[(256 * 4) + 2];               // UART RECV BUFFER


            UART.DiscardInBuffer();

            SendBuf[0] = 0x16;
            SendBuf[1] = (byte)((Adr & 0xF) | 0x10);
            SendBuf[2] = (byte)((Num - 1) & 0xFF);
            SendBuf[3] = (byte)(SendBuf[1] ^ SendBuf[2]);
            UART.Write(SendBuf, 0, 4);

            Rest = ((Num * 4) + 2);
            i = 0;
            while (true)
            {
                try
                {
                    Ret = UART.Read(RecvBuf, (int)i, (int)Rest);
                }
                catch (System.TimeoutException)
                {
                    return P3DBG_NG;

                }
                if (Ret == 0)
                {
                    return P3DBG_NG;
                }
                i = i + (uint)Ret;
                Rest = Rest - (uint)Ret;
                if (Rest == 0) break;
                System.Threading.Thread.Sleep(50);
            }
            for (i = 0, j = 0; j < Num; i = i + 4, j++)
            {
                Tmp = ((uint)RecvBuf[i + 1] << 24) & 0xFF000000;
                Tmp |= ((uint)RecvBuf[i + 2] << 16) & 0x00FF0000;
                Tmp |= ((uint)RecvBuf[i + 3] << 8) & 0x0000FF00;
                Tmp |= ((uint)RecvBuf[i + 4] << 0) & 0x000000FF;
                Data[j] = Tmp;
            }
            return P3DBG_OK;
        }
        //=　[全レジスタ読み出し] ==================================
        public uint CoreRegReadAll(uint PID, ref uint[] Data)
        {
            uint CntrRegSet;
            uint CntrRegOrg;
            uint Psm3CoreReg;
            uint i;

            CntrRegSet = 0;
            CntrRegOrg = 0;
            Psm3CoreReg = 0;

            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if(DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;
            // ContReg設定
            CntrRegSet = ClrBit(CntrRegOrg, AccessModeBit | PIDBit | P1StepGoBit | P0StepGoBit);
            CntrRegSet = SetBit(CntrRegSet, P0DebugReqBit | P1DebugReqBit | PID);
            DbgWrite(CntrReg, CntrRegSet);
            CntrRegSet = SetBit(CntrRegSet, P0RegRqBit | P1RegRqBit);
            DbgWrite(CntrReg, CntrRegSet);
            // ADRESS設定
            // レジスタ読み出し
            for (i = 0; i < 19; i++)
            {
                DbgWrite(AddrReg, i);
                if(DbgRead(DataReg, ref Psm3CoreReg) != P3DBG_OK) return P3DBG_NG;
                Data[i] = Psm3CoreReg;
            }
            // ContReg設定を元にもどす
            CntrRegSet = ClrBit(CntrRegSet, P0RegRqBit | P1RegRqBit);
            DbgWrite(CntrReg, CntrRegSet);
            DbgWrite(CntrReg, CntrRegOrg);

            // SerialPort CLOSE
            DbgClose();
            return P3DBG_OK;
        }
        //=　[レジスタ読み出し] ====================================
        public uint CoreRegRead(uint PID, uint Adr, ref uint Data)
        {
            uint CntrRegSet;
            uint CntrRegOrg;
            uint Psm3CoreReg;

            CntrRegSet = 0;
            CntrRegOrg = 0;
            Psm3CoreReg = 0;

            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;
            // ContReg設定
            CntrRegSet = ClrBit(CntrRegOrg, AccessModeBit | PIDBit | P1StepGoBit | P0StepGoBit);
            CntrRegSet = SetBit(CntrRegSet, P0DebugReqBit | P1DebugReqBit | PID);
            DbgWrite(CntrReg, CntrRegSet);
            CntrRegSet = SetBit(CntrRegSet, P0RegRqBit | P1RegRqBit);
            DbgWrite(CntrReg, CntrRegSet);
            // ADRESS設定
            // レジスタ読み出し
            DbgWrite(AddrReg, Adr);
            if (DbgRead(DataReg, ref Psm3CoreReg) != P3DBG_OK) return P3DBG_NG;
            Data = Psm3CoreReg;
            // ContReg設定を元にもどす
            CntrRegSet = ClrBit(CntrRegSet, P0RegRqBit | P1RegRqBit);
            DbgWrite(CntrReg, CntrRegSet);
            DbgWrite(CntrReg, CntrRegOrg);
            // SerialPort CLOSE
            DbgClose();
            return P3DBG_OK;
        }
        //=　[レジスタ書き込み] ====================================
        public uint CoreRegWrite(uint PID, uint Adr, uint Data)
        {
            uint CntrRegSet;
            uint CntrRegOrg;

            CntrRegSet = 0;
            CntrRegOrg = 0;

            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;
            // ContReg設定
            CntrRegSet = ClrBit(CntrRegOrg, AccessModeBit | PIDBit | P1StepGoBit | P0StepGoBit);
            CntrRegSet = SetBit(CntrRegSet, P0DebugReqBit | P1DebugReqBit | PID);
            DbgWrite(CntrReg, CntrRegSet);
            CntrRegSet = SetBit(CntrRegSet, P0RegRqBit | P1RegRqBit);
            DbgWrite(CntrReg, CntrRegSet);
            // ADRESS設定
            DbgWrite(AddrReg, Adr);
            // レジスタ書き込み
            DbgWrite(DataReg, Data);
            // ContReg設定を元にもどす
            CntrRegSet = ClrBit(CntrRegSet, P0RegRqBit | P1RegRqBit);
            DbgWrite(CntrReg, CntrRegSet);
            DbgWrite(CntrReg, CntrRegOrg);

            // SerialPort CLOSE
            DbgClose();
            return P3DBG_OK;
        }
        //=　[memory読み出し] ======================================
        public uint MemRead(uint Adr, ref uint Data)
        {
            uint CntrRegSet;
            uint CntrRegOrg;

            CntrRegSet = 0;
            CntrRegOrg = 0;

            uint MemVal = 0;
            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;
            // ビット設定
            CntrRegSet = ClrBit(CntrRegOrg, PIDBit /*| P1StepGoBit | P0StepGoBit*/);
            CntrRegSet = SetBit(CntrRegSet, /*P0DebugReqBit |*/ AccessModeBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);
            // ADRESS設定
            DbgWrite(AddrReg, Adr);
            // memory読み出し
            if (DbgRead(DataReg, ref MemVal) != P3DBG_OK) return P3DBG_NG;
            Data = MemVal;
            // ContReg設定を元にもどす
            DbgWrite(CntrReg, CntrRegOrg);

            // SerialPort CLOSE
            DbgClose();
            return P3DBG_OK;
        }
        //=　[memory書き込み] ======================================
        public uint MemWrite(uint Adr, uint Data)
        {
            uint CntrRegSet;
            uint CntrRegOrg;

            CntrRegSet = 0;
            CntrRegOrg = 0;

            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;
            // ビット設定
            CntrRegSet = ClrBit(CntrRegOrg, PIDBit);
            CntrRegSet = SetBit(CntrRegSet, AccessModeBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);
            // ADRESS設定
            DbgWrite(AddrReg, Adr);
            // メモリ書き込み
            DbgWrite(DataReg, Data);
            // ContReg設定を元にもどす
            DbgWrite(CntrReg, CntrRegOrg);
            // SerialPort CLOSE
            DbgClose();
            return P3DBG_OK;
        }
        //=　[memory(16ワード連続)読み出し] ========================
        public uint MemRead16(uint Adr, ref uint[] Data)
        {
            uint CntrRegSet;
            uint CntrRegOrg;
            uint MemVal = 0;
            uint i;

            CntrRegSet = 0;
            CntrRegOrg = 0;
            MemVal = 0;
            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;
            // ビット設定
            CntrRegSet = ClrBit(CntrRegOrg, PIDBit);
            CntrRegSet = SetBit(CntrRegSet, AccessModeBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);
            // ADRESS設定
            // 読み出し
            for (i = 0; i < 16; i++)
            {
                DbgWrite(AddrReg, Adr + (i * 4));
                if (DbgRead(DataReg, ref MemVal) != P3DBG_OK) return P3DBG_NG;
                Data[i] = MemVal;
            }
            // ContReg設定を元にもどす
            DbgWrite(CntrReg, CntrRegOrg);

            // SerialPort CLOSE
            DbgClose();
            return P3DBG_OK;
        }
        //=　[memoryブロック読み出し] ========================
        public uint MemBlockRead(uint TopAdr, uint NumOfWord, ref uint[] Data)
        {
            uint CntrRegSet;
            uint CntrRegOrg;
            uint MemVal = 0;
            uint i;

            CntrRegSet = 0;
            CntrRegOrg = 0;
            MemVal = 0;
            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;
            // ビット設定
            CntrRegSet = ClrBit(CntrRegOrg, PIDBit);
            CntrRegSet = SetBit(CntrRegSet, AccessModeBit | AddrIncMode | ClrChkSumBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);
            // ビット設定
            CntrRegSet = ClrBit(CntrRegSet, ClrChkSumBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);
            // ADRESS設定
            DbgWrite(AddrReg, TopAdr);
            // 読み出し
            for (i = 0; i < NumOfWord; i++)
            {
                if (DbgRead(DataReg, ref MemVal) != P3DBG_OK) return P3DBG_NG;
                Data[i] = MemVal;
            }
            // ContReg設定を元にもどす
            DbgWrite(CntrReg, CntrRegOrg);

            // SerialPort CLOSE
            DbgClose();
            return P3DBG_OK;
        }
        //=　[memoryブロック読み出し2] ========================
        public uint MemBlockRead2(uint TopAdr, uint NumOfWord, ref uint[] Data)
        {
            uint CntrRegSet;
            uint CntrRegOrg;
            uint MemVal = 0;
            uint CheckSum = 0;
            uint TgtCheckSum = 0;
            uint i;

            CntrRegSet = 0;
            CntrRegOrg = 0;
            MemVal = 0;
            CheckSum = 0;
            TgtCheckSum = 0;
            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;
            // ビット設定
            CntrRegSet = ClrBit(CntrRegOrg, PIDBit);
            CntrRegSet = SetBit(CntrRegSet, AccessModeBit | AddrIncMode | ClrChkSumBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);
            // ビット設定
            CntrRegSet = ClrBit(CntrRegSet, ClrChkSumBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);
            // ADRESS設定
            DbgWrite(AddrReg, TopAdr);
            // 読み出し
            for (i = 0; i < NumOfWord; i++)
            {
                if (DbgRead(DataReg, ref MemVal) != P3DBG_OK) return P3DBG_NG;
                Data[i] = MemVal;
                CheckSum += MemVal;
            }
            if (DbgRead(ChkSumReg, ref TgtCheckSum) != P3DBG_OK) return P3DBG_NG;
            // ContReg設定を元にもどす
            DbgWrite(CntrReg, CntrRegOrg);

            // SerialPort CLOSE
            DbgClose();
            if (CheckSum == TgtCheckSum)
            {
                return P3DBG_OK;
            }
            else
            {
                return P3DBG_NG;
            }
        }
        //=　[memoryブロック読み出し3] ========================
        public uint MemBlockRead3(uint TopAdr, uint NumOfWord, ref uint[] Data)
        {
            uint CntrRegSet;
            uint CntrRegOrg;
            uint CheckSum = 0;
            uint TgtCheckSum = 0;
            uint i;

            CntrRegSet = 0;
            CntrRegOrg = 0;
            CheckSum = 0;
            TgtCheckSum = 0;
            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            DbgRead(CntrReg, ref CntrRegOrg);
            // ビット設定
            CntrRegSet = ClrBit(CntrRegOrg, PIDBit);
            CntrRegSet = SetBit(CntrRegSet, AccessModeBit | AddrIncMode | ClrChkSumBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);
            // ビット設定
            CntrRegSet = ClrBit(CntrRegSet, ClrChkSumBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);
            // ADRESS設定
            DbgWrite(AddrReg, TopAdr);
            // 読み出し
            DbgReadBurst(DataReg, ref Data, NumOfWord);

            DbgRead(ChkSumReg, ref TgtCheckSum);
            // ContReg設定を元にもどす
            DbgWrite(CntrReg, CntrRegOrg);

            // SerialPort CLOSE
            DbgClose();
            for (i = 0; i < NumOfWord; i++)
            {
                CheckSum += Data[i];
            }
            if (CheckSum == TgtCheckSum)
            {
                return P3DBG_OK;
            }
            else
            {
                return P3DBG_NG;
            }
        }
        //=　[memory書き込み(ブロック書込)] ========================
        public uint MemBlockWrite(uint TopAdr, uint NumOfWord, ref uint[] DataArray)
        {
            uint CntrRegSet;
            uint CntrRegOrg;
            uint HostChackSum;
            uint TargetChackSum;
            uint Ret;
            uint i;

            CntrRegSet = 0;
            CntrRegOrg = 0;
            HostChackSum = 0;
            TargetChackSum = 0;

            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;
            // ビット設定
            CntrRegSet = ClrBit(CntrRegOrg, PIDBit);
            CntrRegSet = SetBit(CntrRegSet, AccessModeBit | AddrIncMode | ClrChkSumBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);
            // ビット設定
            CntrRegSet = ClrBit(CntrRegSet, ClrChkSumBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);
            // ADRESS設定
            DbgWrite(AddrReg, TopAdr);
            // メモリ書込み
            for (i = 0; i < NumOfWord; i++)
            {
                //DbgWrite(AddrReg, (TopAdr+ i*4));
                DbgWrite(DataReg, DataArray[i]);
                HostChackSum += DataArray[i];
            }
            // チェックサム確認
            //Ret = P3DBG_OK;

            if (DbgRead(ChkSumReg, ref TargetChackSum) != P3DBG_OK) return P3DBG_NG;
            if (TargetChackSum == HostChackSum)
            {
                Ret = P3DBG_OK;
            }
            else
            {
                Ret = P3DBG_NG;
            }
            
            // ContReg設定を元にもどす
            DbgWrite(CntrReg, CntrRegOrg);

            // SerialPort CLOSE
            DbgClose();
            return Ret;
        }
        //=　[memory書き込み(ブロック書込)2] ========================
        public uint MemBlockWrite2(uint TopAdr, uint NumOfWord, ref uint[] DataArray)
        {
            uint CntrRegSet;
            uint CntrRegOrg;
            uint HostChackSum;
            uint TargetChackSum;
            uint Ret;
            uint i;

            CntrRegSet = 0;
            CntrRegOrg = 0;
            HostChackSum = 0;
            TargetChackSum = 0;

            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;
            // ビット設定
            CntrRegSet = ClrBit(CntrRegOrg, PIDBit);
            CntrRegSet = SetBit(CntrRegSet, AccessModeBit | AddrIncMode | ClrChkSumBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);
            // ビット設定
            CntrRegSet = ClrBit(CntrRegSet, ClrChkSumBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);
            // ADRESS設定
            DbgWrite(AddrReg, TopAdr);
            // メモリ書込み
            DbgWriteBurst(DataReg, ref DataArray, NumOfWord);
            // チェックサム確認
            for (i = 0; i < NumOfWord; i++) HostChackSum += DataArray[i];

            if (DbgRead(ChkSumReg, ref TargetChackSum) != P3DBG_OK) return P3DBG_NG;
            if (TargetChackSum == HostChackSum)
            {
                Ret = P3DBG_OK;
            }
            else
            {
                Ret = P3DBG_NG;
            }

            // ContReg設定を元にもどす
            DbgWrite(CntrReg, CntrRegOrg);

            // SerialPort CLOSE
            DbgClose();
            return Ret;
        }
        //=　[デバッガ制御レジスタ値取得] ================
        public uint GetDbgCntrReg(uint RegNo, ref uint DbgCntrVal)
        {
            // SerialPort OPEN
            DbgOpen();

            // ContReg現在値取得
            if (DbgRead(RegNo, ref DbgCntrVal) != P3DBG_OK) return P3DBG_NG;

            // SerialPort CLOSE
            DbgClose();
            return P3DBG_OK;
        }
        //=　[デバッガ制御レジスタ値設定] ================
        public uint SetDbgCntrReg(uint RegNo, uint DbgCntrVal)
        {
            // SerialPort OPEN
            DbgOpen();

            // ContReg現在値取得
            DbgWrite(RegNo, DbgCntrVal);

            // SerialPort CLOSE
            DbgClose();
            return P3DBG_OK;
        }
        //=　[プロセッサ状態取得(RUN/STOP)] ========================
        public uint GetProsessorStatus(uint ProcessorId)
        {
            uint CntrRegOrg;

            CntrRegOrg = 0;
            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;

            // SerialPort CLOSE
            DbgClose();
            if (ProcessorId == P0)
            {
                if ((CntrRegOrg & P0DebugEnaBit) != 0) return PROCESSOR_STOP;
                else return PROCESSOR_RUN;
            }
            else if(ProcessorId == P1)
            {
                if ((CntrRegOrg & P1DebugEnaBit) != 0) return PROCESSOR_STOP;
                else return PROCESSOR_RUN;
            }
            else
            {
                return PROCESSOR_STOP;
            }

        }
        //=　[プロセッサ状態取得(BREAK)] ========================
        public uint GetBreakStatus()
        {
            uint CntrRegOrg;

            CntrRegOrg = 0;
            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;

            // SerialPort CLOSE
            DbgClose();
            return (CntrRegOrg & (P0BreakBit | P1BreakBit));
        }
        //=　[プロセッサRUN] ========================
        public uint Run(uint P0Run, uint P1Run) // 引数：0以外なら該当プロセッサをRUNする
        {
            uint RunBit;
            uint CntrRegOrg;

            CntrRegOrg = 0;

            if (P0Run != 0) RunBit = P0DebugReqBit;
            else RunBit = 0;
            if (P1Run != 0) RunBit |= P1DebugReqBit;

            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;

            // ContReg設定
            CntrRegOrg = ClrBit(CntrRegOrg, RunBit | ClrCashBit);
            DbgWrite(CntrReg, CntrRegOrg);
           
            // SerialPort CLOSE
            DbgClose();

            return P3DBG_OK;

        }
        //=　[プロセッサSTOP] ========================
        public uint Stop(uint P0Stop, uint P1Stop) // 引数：0以外なら該当プロセッサをSTOPする
        {
            uint StopBit;
            uint CntrRegOrg;

            if (P0Stop != 0) StopBit = P0DebugReqBit;
            else StopBit = 0;
            if (P1Stop != 0) StopBit |= P1DebugReqBit;

            CntrRegOrg = 0;
            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;
            CntrRegOrg = ClrBit(CntrRegOrg, P0StepGoBit | P1StepGoBit);
            CntrRegOrg = SetBit(CntrRegOrg, StopBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegOrg);

            // SerialPort CLOSE
            DbgClose();

            return P3DBG_OK;

        }
        //=　[プロセッサSTEP] ========================
        public uint Step(uint Cycle)
        {
            uint CntrRegOrg;
            uint CntrRegSet;
            //uint SR;

            //SR = 0;
            CntrRegOrg = 0;
            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;

            // ビット設定
            CntrRegSet = ClrBit(CntrRegOrg, AccessModeBit | AddrIncMode | PIDBit | P1StepGoBit | P0StepGoBit);
            CntrRegSet = SetBit(CntrRegSet, P0DebugReqBit | P1DebugReqBit | PID0);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);

            // ビット設定
            CntrRegSet = ClrBit(CntrRegOrg, AccessModeBit | AddrIncMode | PIDBit | P1StepGoBit | P0StepGoBit);
            CntrRegSet = SetBit(CntrRegSet, P0DebugReqBit | P1DebugReqBit | PID1);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);

            // P0 StepReg設定
            DbgWrite(P0StepCycReg, Cycle);
            // P1 ContReg設定
            DbgWrite(P1StepCycReg, Cycle);
            // ContReg設定
            CntrRegOrg = SetBit(CntrRegOrg, P0StepGoBit | P1StepGoBit);
            DbgWrite(CntrReg, CntrRegOrg);

            // SerialPort CLOSE
            DbgClose();

            return P3DBG_OK;

        }
        //=　[プロセッサSTEP解除] ========================
        public uint StepModeOff()
        {
            uint CntrRegOrg;

            CntrRegOrg = 0;
            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;
            CntrRegOrg = ClrBit(CntrRegOrg, P0StepGoBit | P1StepGoBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegOrg);

            // SerialPort CLOSE
            DbgClose();

            return P3DBG_OK;
        }
        //=　[キャッシュリセット] ====================================
        public uint CashResetForce()
        {
            uint CntrRegSet;
            uint CntrRegOrg;

            CntrRegSet = 0;
            CntrRegOrg = 0;

            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;

            // ビット設定
            CntrRegSet = SetBit(CntrRegOrg, P0DebugReqBit | P1DebugReqBit | ClrCashBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);

            // ContReg設定を元にもどす
            DbgWrite(CntrReg, CntrRegOrg);
            // SerialPort CLOSE
            DbgClose();
            return P3DBG_OK;
        }
        //=　[キャッシュ強制OFF] ====================================
        public uint CashOffForce()
        {
            uint CntrRegSet;
            uint CntrRegOrg;

            CntrRegSet = 0;
            CntrRegOrg = 0;

            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;

            // ビット設定
            CntrRegSet = SetBit(CntrRegOrg, ClrCashBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);

            // SerialPort CLOSE
            DbgClose();
            return P3DBG_OK;
        }
        //=　[キャッシュ強制OFF解除] ====================================
        public uint CashOnForce()
        {
            uint CntrRegSet;
            uint CntrRegOrg;

            CntrRegSet = 0;
            CntrRegOrg = 0;

            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;
            // ビット設定
            CntrRegSet = ClrBit(CntrRegOrg, ClrCashBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);

            // SerialPort CLOSE
            DbgClose();
            return P3DBG_OK;
        }
        //=　[プロセッサリセット] ====================================
        public uint Reset()
        {
            uint CntrRegSet;
            uint CntrRegOrg;

            CntrRegSet = 0;
            CntrRegOrg = 0;

            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;
            // ビット設定
            CntrRegSet = SetBit(CntrRegOrg, PRstBit);
            // ContReg設定
            DbgWrite(CntrReg, CntrRegSet);

            // SerialPort CLOSE
            DbgClose();
            return P3DBG_OK;
        }
        //=　[Bubbleチェック] ====================================
        public uint IsBubble(uint PNo)
        {
            uint CntrRegOrg;
            uint Ret;

            Ret = 0;
            CntrRegOrg = 0;

            // SerialPort OPEN
            DbgOpen();
            // ContReg現在値取得
            if (DbgRead(CntrReg, ref CntrRegOrg) != P3DBG_OK) return P3DBG_NG;

            if (PNo == 0)
            {
                if ((CntrRegOrg & P0BubbleBit) != 0) Ret = 1;
                else Ret = 0;
            }
            else
            {
                if ((CntrRegOrg & P1BubbleBit) != 0) Ret = 1;
                else Ret = 0;
            }

            // SerialPort CLOSE
            DbgClose();
            return Ret;
        }
    }
}
